# Rhye's and Fall of Civilization - Barbarian units and cities

import CvUtil
import PyHelpers	# LOQ
#import Popup
#import cPickle as pickle
from RFCUtils import *
from Consts import *
from StoredData import data

from Events import handler
from Core import *
from Modifiers import *

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer	# LOQ

# also colonize for the player (simulates private sector)
constMicromodColonizeForHumanPlayer = False				# default=False
constMicromodAdjustmentsForColonies = False				# default=False (unfortunately it slows the game down by about 50%)
constMicromodAdjustmentsForTech = True					# default=True 
constMicromodCrusadeEnabled = True						# default=True

# Spawning cities (Leoreth)
# Year, coordinates, owner, name, population, unit type, unit number, religions, forced spawn
tMinorCities = (

# ANCIENT ERA
# Note: these cities do not get retroactively spawned in 600AD or 1700AD

# ancient central asia (independent)
(-2000, (85, 47), iIndependent, 'Afrasiyab', 1, iArcher, 1), 	# Samarkand
(-75, (89, 46), iBarbarian, 'Kashgar', 2, iArcher, 1),			# Kashgar

# ancient europe (independent + celts)
(-770, (59, 47), iCelts, 'Melpum', 2, iArcher, 2),				# Milan
(-500, (62, 49), iCelts, 'Vinadoba', 2, -1, -1),				# Hallstatt / Vienna
(-450, (56, 48), iCelts, 'Lugodunon', 2, -1, -1),				# Lyon
(-250, (55, 50), iCelts, 'Lutetia', 2, -1, -1),					# Paris
(-300, (53, 48), iCelts, 'Burdigala', 2, -1, -1),				# Bordeaux
(-300, (65, 45), iCelts, 'Singidunum', 2, -1, -1),				# Belgrade
(-50, (51, 50), iCelts, 'Brest', 2, -1, -1),					# Brest
(-100, (65, 48), iIndependent2, 'Sarmizegetusa', 2, iArcher, 1),# Dacians, also Buda

# ancient MENA (independent)
(-3000, (80, 40), iIndependent2, 'Shushan', 1, iArcher, 1), 	# Susa (Ishfahan)
(-3000, (73, 38), iIndependent, 'Yerushalayim', 2, iArcher, 3),	# Jerusalem
(-2500, (69, 30), iIndependent2, 'Buhen', 2, iMilitia, 2),		# Kerma culture / Khartoum / Buhen
(-1750, (72, 43), iIndependent, 'Ankuwash', 2, iArcher, 2),		# Ankara / Konya / Hittite
(-2500, (80, 42), iBarbarian, 'Ray', 1, -1, -1),				# Tabriz. Barbarian raiders into mesopotamia, babylon hammer sink.
(-190, (77, 45), iIndependent2, 'Artashat', 2, -1, -1),			# Artaxata -> Tiflis
(-1650, (70, 42), iIndependent, 'Troy', 1, -1, -1),				# Wilusa-Patara composite / Ephesus / Izmir
(100, (76, 30), iIndependent, "Sana'a", 2, iArcher, 2),			# Sana'a

# egypt expansion
(-2500, (67, 36), iEgypt, 'Ra-Kedet', 1, -1, -1),				# Alexandria
(-1300, (70, 36), iEgypt, 'Tanis', 1, -1, -1),					# Tanis / Djanet (encourage meditaraneean - indian trade, as historical. )
																# also historical : egypt doesn't sign open borders easily.
																
# persian expansion
(-714, (78, 43), iPersia, 'Tabriz', 1, -1, -1),					# Tabriz
(-550, (83, 45), iPersia, 'Mouru', 1, -1, -1),					# Alexandria Margiana / Merv
(-550, (84, 42), iPersia, 'Haraiva', 1, -1, -1),				# Herat
(-522, (81, 34), iPersia, 'Maka', 1, -1, -1),					# Maka / Dubai
(-650, (85, 36), iPersia, 'Gwadar', 1, -1, -1),					# Gwadar / Karachi
(-450, (87, 42), iPersia, 'Taxila', 1, -1, -1),					# Taxila

# greece expansion
(-1800, (69, 39), iIndependent, 'Knossos', 1, -1, -1),			# Knossos (Minoans) / Iraklion, flipped.
(-1200, (70, 42), iGreece, 'Ephesus', 1, -1, -1),				# Ephesus Izmir
(-1550, (68, 45), iIndependent2, 'Byzantium', 1, -1, -1),		# Byzantium + early Greek blocker
(-800, (62, 42), iGreece, 'Neapolis', 1, iWorker, 1),			# Neapolis (Magna Grecia)
(-734, (60, 40), iGreece, 'Syracuse', 1, iWorker, 1),			# Syracuse (Magna Grecia) / Palermo
(-750, (75, 44), iGreece, 'Trapezon', 1, -1, -1),				# Trapezon
(-350, (67, 36), iGreece, 'Alexandria', 1, -1, -1),				# Alexandria

# ancient north africa (indepedent because they were colonized/vassalized by greeks, carthaginians, romans, egyptians, etc)
(-1000, (64, 36), iIndependent, 'Cyrene', 2, iSpearman, 3),		# Cyrene

# carthaginian expansions
(-600, (54, 38), iCarthage, 'Rusadir', 1, iSpearman, 2),	# Mauretanians / Wahran 
(-500, (51, 41), iCarthage, 'Gades', 1, -1, -1),				# Gades / Seville / Cordoba
(-450, (54, 42), iCarthage, 'Carthago Nova', 1, -1, -1),		# Valencia
(-600, (60, 37), iCarthage, 'Tripolis', 1, -1, -1),				# Tripolis

# roman expansion
(-250, (58, 42), iRome, 'Caralis', 1, iWorker, 1),				# Cagliari
(-250, (61, 47), iRome, 'Aquileia', 1, iLegion, 4),				# Venice (v. early but rome needs cities in its core)
(-380, (56, 46), iRome, 'Massilia', 1, iLegion, 3),				# Marseilles
(-360, (55, 44), iRome, 'Taracco', 1, iLegion, 3),				# Barcelona
(-50, (52, 44), iRome, 'Toletum', 1, iLegion, 3),				# Madrid
(-200, (49, 43), iRome, 'Felicitas Julia', 1, iLegion, 3),		# Lisbon
(-200, (62, 49), iRome, 'Vindobona', 1, iLegion, 4),			# Wien / Vienna, much earlier AI loves to settle the area randomly
(50, (53, 54), iRome, 'Londinium', 1, -1, -1),					# London
(50, (51, 37), iRome, 'Volubilis', 1, -1, -1),					# Marrakesh (grew into a major city under the romans)
(20, (59, 51), iRome, 'Moguntiacum', 1, iLegion, 4),			# Mainz
(-75, (65, 45), iRome, 'Naissium', 1, iLegion, 4),				# Belgrade

# ancient china
(-2070, (103, 44), iIndependent2, 'Luoyang', 1, -1, -1),		# before Han dynasty, will be flipped
(-1500, (103, 42), iChina, 'Wuhan', 1, -1, -1),					# Wuhan
(-1000, (102, 47), iIndependent, 'Zhongdu', 2, iSpearman, 1),	# Beijing
(-500, (99, 41), iIndependent2, 'Chengdu', 2, iArcher, 1),		# Chengdu
(-400, (105, 43), iChina, 'Nanjing', 1, iArcher, 1),			# Nanjing
(-300, (105, 49), iBarbarian, 'Simiyan hoton', 2, iChariot, 2),	# Shenyang
(-1122, (108, 48), iIndependent2, 'Pyongyang', 2, iSpearman, 3),# Pyong Yang
(-1500, (105, 39), iNative, 'Guangzhou', 2, iArcher, 1),		# Guangzhou / Baiyue
(-100, (95, 47), iBarbarian, 'Dunhuang', 2, iArcher, 1),		# Dunhuang
(-2000, (107, 42), iBarbarian, 'Hangzhou', 2, -1, -1),			# Hangzhou (early, chinese hammer sink)
(-2000, (107, 44), iBarbarian, 'Shanghai', 2, -1, -1),			# Shanghai (early, chinese hammer sink)
(-800, (101, 41), iIndependent2, 'Chongqing', 5, iSpearman, 3),	# Chongqing

# ancient india
(-1600, (90, 40), iIndependent, 'Indraprastha', 1, iMilitia, 1),	# Delhi , flipped
(-700, (90, 37), iIndia, 'Mathura', 1, -1, -1),						# Mathura / Bhopal
(-700, (95, 37), iIndia, 'Kolkata', 1, -1, -1),						# Kolkata
(-325, (92, 32), iIndependent2, 'Kanchipuram', 2, iArcher, 1),		# Chennai
(-300, (90, 31), iIndependent, 'Vijayanagara', 1, iWarElephant, 1),	# Vijayanagara
(-150, (94, 34), iIndependent2, 'Visakhapatnam', 2, -1, -1),		# Visakhapatnam
(140, (87, 40), iIndia, 'Lahore', 1, -1, -1),						# Lahore
(325, (88, 34), iIndependent, 'Pune', 2, iArcher, 1),				# Mumbai

# other
(-250, (19, 35), iNative, 'Danibaan', 2, iHolkan, 1),	# Monte Albn
(100, (18, 37), iBarbarian, 'Tolan', 2, iJaguar, 2),		# Teotihuacan
(107, (98, 36), iIndependent2, 'Pagan', 2, -1, -1),			# Pagan




# Note: - most of the old world was populated into many small and independent kingdoms, so this list just populates these places
#       - natives/barbarians are used for places where the level of civilization was tribal/migratory
#       - using the most recognizable name (e.g. La Paz)
#       - this mechanism also simulates colonial range and independent elements of the kingdom (many colonists wanted to leave the old world for various reasons)
#       - most cities start at lvl 2 to minimize city razing chances


# europe - north of alps
(910, (61, 63), iIndependent2, 'Trondheim', 2, iHuscarl, 1),	# Trondheim
(610, (58, 61), iIndependent2, 'Bergen', 2, iHuscarl, 1),
(810, (62, 49), iIndependent, 'Vienna', 2, iCrossbowman, 4),	# Wien
(810, (59, 54), iIndependent, 'Hamburg', 2, iCrossbowman, 5),	# Hamburg
(750, (57, 53), iIndependent2, 'Amsterdam', 3, iCrossbowman, 2),# Amsterdam
(840, (65, 48), iIndependent2, 'Buda', 3, iHorseArcher, 5),				# Budapest
(610, (56, 48), iIndependent, 'Lyon', 6, iCrossbowman, 5),		# Lyon
(610, (53, 48), iIndependent2, 'Bordeaux', 4, iCrossbowman, 3),		# Bordeaux
(710, (51, 50), iIndependent2, 'Brest', 2, iCrossbowman, 3),	# Brest
(810, (62, 53), iIndependent2, 'Brandenburg', 2, iCrossbowman, 2),			# Brandenburg
(810, (62, 51), iIndependent, 'Prague', 3, iCrossbowman, 4),			# Prague
(850, (66, 53), iIndependent, 'Warsaw', 2, iCrossbowman, 3),			# Warsaw
(850, (65, 55), iIndependent2, 'Koenigsberg', 2, iCrossbowman, 2),			# Koenigsberg

#europe - british isles
(710, (50, 53), iIndependent, 'Plymouth', 1, -1, -1),
(750, (53, 57), iIndependent, 'Newcastle', 1, -1, -1),
(800, (52, 59), iIndependent2, 'Edinburgh', 1, -1, -1),
(750, (51, 55), iIndependent, 'Liverpool', 1, -1, -1),

#europe - south of alps
(610, (70, 42), iByzantium, 'Ephesus', 5, iCrossbowman, -1),			# Ephesus / Izmir
(640, (69, 39), iByzantium, 'Iraklion', 1, -1, -1),					# after arab flip
(610, (72, 45), iByzantium, 'Sinop', 3, iCrossbowman, 1),	
(610, (60, 40), iByzantium, 'Palermo', 4, -1, -1),			# Palermo
(610, (59, 47), iIndependent, 'Milano', 6, iCrossbowman, 2),			# Milano
(610, (61, 47), iIndependent, 'Venice', 4, iCrossbowman, 2),			# Venice
(610, (49, 45), iIndependent2, 'Santiago', 2, iCrossbowman, 3),			# Santiago
(610, (58, 42), iIndependent2, 'Cagliari', 1, -1, -1),	

# europe - russia
(840, (69, 52), iIndependent, 'Kyiv', 4, iCrossbowman, 3),		# Kiev
(840, (69, 58), iIndependent2, 'St Petersburg', 2, iCrossbowman, 2),			# St Petersburg
(840, (67, 59), iIndependent2, 'Helsinki', 2, iCrossbowman, 1),			# Helsinki
(840, (70, 55), iIndependent2, 'Novgorod', 2, iCrossbowman, 2),			# Novgorod
(840, (76, 51), iBarbarian, 'Sarai', 2, iArcher, 2),			# Sarai Batu
(840, (78, 50), iBarbarian, 'Astrahan', 2, iArcher, 2),			# Astrahan
(840, (72, 51), iIndependent, 'Kharkiv', 2, iCrossbowman, 2),			
(840, (74, 49), iBarbarian, 'Rostov', 2, iArcher, 2),			# Rostov
(840, (69, 49), iIndependent2, 'Odessa', 2, iCrossbowman, 3),			# Odessa
(840, (73, 57), iIndependent, 'Jaroslavl', 2, iCrossbowman, 1),		

# steppes - barbarians and turks
(900, (80, 58), iBarbarian, 'Perm', 2, iArcher, 2),			# Perm
(900, (83, 55), iBarbarian, 'Cheljabinsk', 2, iArcher, 1),			# Cheljabinsk
(900, (77, 56), iBarbarian, 'Kazan', 2, iArcher, 1),				# Kazan
(900, (78, 53), iBarbarian, 'Samara', 2, iArcher, 1),
(610, (83, 50), iTurks, 'Chelkar', 2, iHorseArcher, 1),
(610, (95, 53), iBarbarian, 'Kyzyl', 2, iHorseArcher, 1),
(610, (92, 50), iTurks, 'Urumqi', 2, iHorseArcher, 1),	

# mena
(610, (68, 32), iIndependent2, 'Luxor', 4, iCrossbowman, 3),			# Luxor
(610, (69, 30), iIndependent, 'Khartoum', 3, iCrossbowman, 2),			# Khartoum
(610, (58, 39), iIndependent, 'Tunis', 3, iCrossbowman, 1),			# Tunis
(610, (56, 39), iIndependent, 'Al Jazzir', 3, iCrossbowman, 1),		# Al Jazzir
(610, (54, 38), iIndependent, 'Wahran', 3, iCrossbowman, 1),		# Wahran
(610, (80, 40), iIndependent2, 'Ishfahan', 6, iCrossbowman, 3),
(610, (80, 42), iIndependent2, 'Tahran', 6, iCrossbowman, 2),
(610, (78, 43), iIndependent2, 'Tabriz', 7, iCrossbowman, 2),
(610, (77, 45), iIndependent2, 'Tiflis', 2, iCrossbowman, 2),
(610, (51, 37), iIndependent, 'Rabat', 2, iCrossbowman, 2),	# Rabat, to resist the moors

# arabia fast expansion
(630, (81, 34), iArabia, 'Dubai', 1, -1, -1),
(630, (78, 36), iArabia, 'Al-Damman', 1, iCamelArcher, 1),
(630, (79, 31), iArabia, 'Salalah', 1, -1, -1),
(630, (74, 35), iArabia, 'Medina', 1, iCamelArcher, 1),

# india
(610, (87, 42), iIndependent2, 'Peshawar', 3, iCrossbowman, 2),
(610, (85, 36), iIndependent2, 'Karachi', 3, iCrossbowman, 2),
(610, (94, 34), iIndependent, 'Visakhapatnam', 3, iCrossbowman, 1),

# japan
(600, (116, 47), iJapan, 'Edo', 3, iArcher, 1),				
(600, (115, 45), iJapan, 'Nagoya', 3, iArcher, 1),			
(600, (115, 51), iNative, 'Sapporo', 2, iArcher, 6),		
(600, (112, 41), iJapan, 'Kagoshima', 2, iArcher, 1),
(600, (111, 43), iJapan, 'Fukuoka', 2, iArcher, 1),

# china
(610, (96, 43), iBarbarian, 'Rasa', 2, iKhampa, 1),			# Lhasa
(610, (107, 53), iBarbarian, 'Harbin', 2, iCrossbowman, 2),		
(610, (105, 52), iBarbarian, 'Chanchun', 2, iCrossbowman, 2),	
(610, (108, 39), iNative, 'Taibei', 1, iArcher, 1),
(610, (101, 39), iNative, 'Kunming', 2, iCrossbowman, 2),
(610, (101, 41), iNative, 'Chongqing', 5, iCrossbowman, 2),
(610, (99, 47), iChina, 'Baotou', 1, iCrossbowman, 1),
(610, (105, 45), iChina, 'Qingdao', 1, iCrossbowman, 1),
(610, (107, 44), iChina, 'Shanghai', 1, iCrossbowman, 1),
(610, (103, 42), iChina, 'Wuhan', 4, iCrossbowman, 1),
(610, (111, 51), iBarbarian, 'Haishenwai', 2, iCrossbowman, 3),
(1705, (104, 46), iChina, 'Tianjin', 9, iMusketeer, 1),		# 1700 china
(1705, (105, 42), iChina, 'Nanjing', 11, iMusketeer, 1),	# 1700 china

# south east asia, post indonesia
(700, (109, 26), iNative, 'Makassar', 1, iArcher, 1),		# Makassar
(670, (104, 25), iNative, 'Jakarta', 3, iArcher, 1),
(680, (106, 25), iNative, 'Surabaya', 2, iArcher, 1),
(700, (106, 29), iNative, 'Bandar Seri Begawan', 1, iArcher, 1),
(700, (108, 34), iIndependent2, 'Manila', 2, iArcher, 1),	# independent so wardec from conquerors
(700, (110, 31), iIndependent2, 'Davao', 1, iArcher, 1),
(700, (98, 30), iNative, 'Medan', 1, iArcher, 1),
(700, (113, 25), iNative, 'Ternate', 1, iArcher, 1),
(700, (101, 29), iIndependent, 'Singapura', 2, iCrossbowman, 2),
(610, (101, 33), iIndependent, 'Bangkok', 7, iCrossbowman, 4),
(610, (101, 37), iIndependent2, 'Hanoi', 3, iCrossbowman, 4),	# Hanoi
(610, (104, 33), iIndependent, 'Saigon', 4, iCrossbowman, 4),	# Saigon

# pre - conquistadors
(610, (18, 37), iBarbarian, 'Mexico City', 4, -1, -1),
(610, (31, 20), iNative, 'La Paz', 1, iArcher, 1),			# la Paz	#pre-ince civs (chan chan, nazca, etc)
(610, (24, 28), iNative, 'Cali', 1, iArcher, 2),			# Cali
(610, (25, 25), iNative, 'Quito', 1, iArcher, 2),			# Quito
(610, (27, 29), iNative, 'Bogota', 1, iArcher, 3),		# Bogota
(610, (30, 16), iNative, 'Coqimpu', 1, iArcher, 1),		# Coqimpu

# west africa
(610, (62, 20), iNative, 'Mbanza Kongo', 1, iPombos, 1),		# Mbanza Kongo
(610, (64, 22), iNative, 'Kinshasa', 1, iPombos, 1),			# Kongo
(610, (51, 30), iNative, 'Kembi Saleh', 1, iArcher, 1),			#kingdom of ghana
(700, (63, 30), iNative, 'Kanem Bornu', 1, iPombos, 2),			#kanem bornu
(700, (48, 31), iNative, 'Dakar', 1, iPombos, 1),					#ghana
(900, (59, 29), iNative, 'Kano', 1, iPombos, 2),				#kano kingdom
(1000, (56, 34), iIndependent, 'Tamanrasset', 1, iCrossbowman, 2),	#tamanrasset, trade between mediteraneean and west africa

# east africa
(610, (71, 28), iIndependent, 'Addis Ababa', 2, iCrossbowman, 5),	# Addis Ababa + never conquered
(910, (75, 26), iIndependent, 'Muqdisho', 2, iCrossbowman, 2),	# Mogadishu
(1060, (71, 17), iNative, 'Quelimane', 1, iArcher, 2),				# Quelimane
(1110, (71, 20), iNative, 'Dar es Salaam', 1, iArcher, 2),			# Dar es Salaam
(1200, (68, 18), iNative, 'Zimbabwe', 1, iArcher, 2),				#zimbabwe

# COLONIZERS. This helps the ai a lot by providing both historic and constant expansion.
# Dates are gameplay oriented, so sometimes earlier than history.
# Also models proper colonization (capacity, range, technology, e.g. rainforest)

#swedish expansion
(1000, (45, 62), iVikings, 'Reykjavik', 1, -1, -1),

# russian expansion
(1550, (81, 55), iRussia, 'Yekaterinburg', 1, -1, -1),		# over the urals
(1550, (88, 55), iRussia, 'Omsk', 3, -1, -1),
(1580, (91, 55), iRussia, 'Novosibirsk', 3, -1, -1),
(1580, (85, 58), iRussia, 'Tyumen', 3, -1, -1),
(1580, (91, 59), iRussia, 'Tomsk', 3, -1, -1),
(1620, (94, 56), iRussia, 'Krasnoyarsk', 3, -1, -1),		# mid siberia
(1620, (96, 57), iRussia, 'Bratsk', 1, -1, -1),
(1620, (99, 56), iRussia, 'Ust-Kut', 1, -1, -1),
(1650, (98, 53), iRussia, 'Irkutsk', 3, -1, -1),
(1650, (101, 53), iRussia, 'Ulan-Udeh', 1, -1, -1),
(1675, (102, 57), iRussia, 'Lensk', 1, -1, -1),				# far east
(1675, (106, 58), iRussia, 'Jakutsk', 3, -1, -1),
(1675, (111, 58), iRussia, 'Okhotsk', 1, -1, -1),
(1675, (114, 59), iRussia, 'Magadan', 1, -1, -1),
(1675, (111, 54), iRussia, 'Khabarovsk', 3, iCaravel, 1),	#far east explorers
(1675, (119, 55), iRussia, 'Petropavlovsk', 1, -1, -1),
(1800, (101, 61), iRussia, 'Zhigansk', 1, -1, -1),			# tundra expansion
(1800, (69, 64), iRussia, 'Murmansk', 1, -1, -1),
(1800, (94, 64), iRussia, 'Khatanga', 1, -1, -1),
(1800, (84, 62), iRussia, 'Novy Port', 1, -1, -1),
(1800, (80, 62), iRussia, 'Vorkuta', 1, -1, -1),
(1800, (119, 63), iRussia, 'Pevek', 1, -1, -1),

(1750, (10, 56), iRussia, 'Sitka', 1, -1, -1),	# alaska

# carribean partition + explorers
(1575, (29, 37), iFrance, 'Haiti', 1, iCaravel, 2),			
(1550, (26, 41), iSpain, 'Miami', 1, iArquebusier, 1),
(1493, (26, 38), iSpain, 'Havana', 2, iGreatStatesman, 1),			
(1492, (32, 37), iSpain, 'Puerto Rico', 1, iCaravel, 3),	
(1590, (33, 33), iEngland, 'Trinidad Tobago', 1, iCaravel, 2),
(1590, (30, 39), iNetherlands, 'Nassau', 1, iCaravel, 2),

# spanish expansion + conquistadors
(750, (54, 42), iSpain, 'Valencia', 4, iHeavyGalley, 2),	# Valencia guaranteed to spain + spanish fleet for mediteraneean
(1400, (47, 36), iSpain, 'Tenerife', 3, iCaravel, 3),		# early spanish explorers

(1490, (19, 35), iSpain, 'Puebla', 3, iConquistador, 6),	# 1st contact + extra army
(1525, (16, 39), iSpain, 'Guadalajara', 3, iConquistador, 3),
(1500, (25, 32), iSpain, 'Panama', 2, iConquistador, 1),
(1500, (28, 32), iSpain, 'Cartagena', 3, iConquistador, 1),
(1530, (31, 31), iSpain, 'Caracas', 3, iConquistador, 1),
(1495, (23, 33), iSpain, 'San Jose', 1, iConquistador, 3),
(1600, (13, 40), iSpain, 'La Paz de California', 1, iArquebusier, 1),

(1500, (26, 22), iSpain, 'Lima', 4, iConquistador, 6),	# inca early
(1525, (29, 11), iSpain, 'Santiago de Chile', 3, iArquebusier, 1),
(1550, (29, 8),  iSpain, 'Conception', 1, iCaravel, 1),
(1550, (36, 12), iSpain, 'Montevideo', 3, -1, -1),
(1550, (34, 11), iSpain, 'Buenos Aires', 3, iGreatStatesman, 1),
(1575, (34, 9), iSpain, 'Bahia Blanca', 1, iCaravel, 1),

(1600, (32, 5), iSpain, 'Santa Cruz', 1, -1, -1),			# la plata late
(1600, (34, 16), iSpain, 'Asuncion', 1, -1, -1),	

(1675, (10, 49), iSpain, 'San Francisco', 1, -1, -1),		# west coast
(1675, (11, 46), iSpain, 'Los Angeles', 1, -1, -1),
(1675, (15, 44), iSpain, 'Phoenix', 1, -1, -1),
(1675, (12, 44), iSpain, 'San Diego', 1, -1, -1),

# english expansion
(990, (49, 57), iEngland, 'Dublin', 1, -1, -1),						# very early english ireland colonization (ai never takes ireland)
(1615, (31, 50), iEngland, 'Boston', 3, iRedcoat, 2),				#america (ahead of time)
(1615, (30, 48), iEngland, 'New York', 4, iRedcoat, 2),
(1615, (28, 47), iEngland, 'Philadelphia', 3, iRedcoat, 2),
(1605, (27, 45), iEngland, 'Jamestown', 1, iRedcoat, 1),
(1655, (25, 43), iEngland, 'Charleston', 1, iRedcoat, 1),			

(1650, (34, 30), iEngland, 'Georgetown', 1, iCaravel, 1),			# south america
(1725, (35, 4), iEngland, 'Stanley', 1, iCaravel, 1),

(1700, (23, 57), iEngland, 'Fort Severn', 1, -1, -1),				# canada
(1650, (27, 51), iEngland, 'Toronto', 3, iGreatStatesman, 1),
(1720, (21, 53), iEngland, 'Winnipeg', 1, iRifleman, 1),
(1775, (17, 56), iEngland, 'Edmonton', 1, -1, -1),
(1750, (17, 53), iEngland, 'Calgary', 1, iRifleman, 1),
(1790, (11, 53), iEngland, 'Vancouver', 1, iRifleman, 1),
(1825, (15, 58), iEngland, 'Fort Nelson', 1, -1, -1),

(1650, (67, 10), iEngland, 'Port Elizabeth', 2, iRedcoat, 2),		# africa (early)
(1650, (69, 13), iEngland, 'Durban', 3, iRedcoat, 2),
(1750, (55, 27), iEngland, 'Accra', 3, iRifleman, 3),
(1750, (59, 26), iEngland, 'Lagos', 3, iRifleman, 3),				
(1750, (50, 28), iEngland, 'Freetown', 1, iRifleman, 3),				

(1700, (118, 13), iEngland, 'Sydney', 3, iGreatStatesman, 1),			# australia
(1700, (115, 10), iEngland, 'Melbourne', 3, iFrigate, 2),
(1725, (112, 12), iEngland, 'Adelaide', 2, -1, -1),
(1725, (118, 16), iEngland, 'Brisbane', 3, -1, -1),
(1725, (104, 13), iEngland, 'Perth', 2, iCaravel, 1),
(1750, (115, 20), iEngland, 'Cairns', 1, -1, -1),
(1750, (110, 21), iEngland, 'Darwin', 1, -1, -1),
(1775, (106, 18), iEngland, 'Broome', 1, -1, -1),
(1800, (111, 16), iEngland, 'Alice Springs', 1, -1, -1),

(1825, (5, 25), iEngland, 'Samoa', 1, iManOfWar, 1),				#pacific (early)
(1825, (17, 17), iEngland, 'Piticairn', 1, iManOfWar, 1),

# french expansion
(1625, (30, 52), iFrance, 'Montreal', 4, iGreatStatesman, 1),		# canada
(1625, (33, 55), iFrance, 'Sept Iles', 1, iCaravel, 1),
(1625, (36, 54), iFrance, 'Saint Johns', 1, iCaravel, 1),
(1625, (35, 41), iFrance, 'Louisbourg', 1, -1, -1),
(1680, (29, 56), iFrance, 'Post-a-la-Baleine', 1, -1, -1),

(1630, (23, 42), iFrance, 'New Orleans', 1, iMusketeer, 2),		# louisiana
# (1660, (23, 46), iFrance, 'Saint Louis', 1, iMusketeer, 2),

(1650, (37, 28), iFrance, 'Cayenne', 1, -1, -1),

(1700, (48, 31), iFrance, 'Dakar', 3, iRifleman, 3),			#early africa
(1750, (75, 17), iFrance, 'Antananarivo', 1, -1, -1),
(1750, (74, 14), iFrance, 'Fort Dauphin', 1, iCaravel, 1),
(1775, (60, 23), iFrance, 'Brazzaville', 1, iRifleman, 3),
(1775, (63, 27), iFrance, 'Douala', 1, iRifleman, 3),
(1775, (53, 26), iFrance, 'Abidjan', 3, iRifleman, 3),

(1800, (122, 19), iFrance, 'New Caledonia', 1, iManOfWar, 1),	#pacific
(1825, (13, 18), iFrance, 'Polynesia', 1, iManOfWar, 1),
(1825, (15, 23), iFrance, 'Marquesas', 1, iManOfWar, 1),

# portugal expansion
(1400, (47, 38), iPortugal, 'Funchal', 3, -1, -1),				# atlantic islands
(1450, (45, 33), iPortugal, 'Praia', 1, iCaravel, 1),	

(1525, (41, 18), iPortugal, 'Rio de Janeiro', 1, iGreatStatesman, 1),
(1525, (38, 17), iPortugal, 'Sao Paulo', 1, iArquebusier, 1),
(1550, (38, 15), iPortugal, 'Porto Alegre', 1, iCaravel, 1),
(1650, (38, 19), iPortugal, 'Belo Horizonte', 1, -1, -1),
(1575, (42, 22), iPortugal, 'Salvador', 1, -1, -1),
(1575, (42, 25), iPortugal, 'Fortaleza', 1, iArquebusier, 1),
(1525, (38, 26), iPortugal, 'Belem', 1, iArquebusier, 1),
(1800, (38, 22), iPortugal, 'Brasilia', 1, -1, -1),

# netherlands expansion
(1600, (30, 48), iNetherlands, 'New Amsterdam', 1, iMusketeer, 2),

(1600, (63, 10), iNetherlands, 'Cape Town', 3, iGreatStatesman, 1),
(1700, (67, 14), iNetherlands, 'Johannesburg', 3, iMusketeer, 2),
(1650, (56, 27), iNetherlands, 'Togo', 1, iMusketeer, 2),
(1650, (54, 16), iNetherlands, 'Saint Helena', 1, -1, -1),
(1650, (30, 4), iNetherlands, 'Kaap Horn', 1, -1, -1),

(1725, (122, 10), iEngland, 'Auckland', 2, -1, -1),			#mixed new zealand
(1750, (121, 6), iNetherlands, 'Christchurch', 2, -1, -1),

# german expansion
(1800, (116, 24), iGermany, 'Port Moresby', 3, iManOfWar, 1),		# v. early oceania
(1825, (119, 27), iGermany, 'Rabaul', 1, iManOfWar, 1),
(1825, (5, 21), iGermany, 'Tonga', 1, iManOfWar, 1),
(1800, (64, 15), iGermany, 'Windhoek', 2, iRifleman, 2),				
(1775, (61, 15), iGermany, 'Walvis Bay', 2, iRifleman, 2),
(1775, (72, 23), iGermany, 'Mombasa', 4, iRifleman, 1),				# Mombasa
(1800, (69, 23), iGermany, 'Nairobi', 2, iRifleman, 1),				# Nairobi

# indy texas
(1700, (20, 42), iIndependent, 'Houston', 1, iArquebusier, 3),

# usa expansion + migration (v. fast)
(1780, (26, 49), iAmerica, 'Detroit', 5, iRifleman, 2),
(1780, (23, 49), iAmerica, 'Chicago', 5, iRifleman, 2),
(1780, (23, 46), iAmerica, 'St Louis', 4, iRifleman, 2),
(1795, (21, 50), iAmerica, 'Minneapolis', 4, iRifleman, 1),
(1795, (20, 44), iAmerica, 'Dallas', 3, iRifleman, 2),
(1800, (18, 48), iAmerica, 'Denver', 3, iRifleman, 1),
(1800, (14, 47), iAmerica, 'Las Vegas', 3, iRifleman, 2),
(1800, (15, 50), iAmerica, 'Idaho Falls', 3, iRifleman, 2),

(1810, (10, 49), iAmerica, 'San Francisco', 4, iRifleman, 2),
(1810, (11, 46), iAmerica, 'Los Angeles', 5, iRifleman, 2),
(1810, (15, 44), iAmerica, 'Phoenix', 4, iRifleman, 2),
(1810, (12, 44), iAmerica, 'San Diego', 4, iRifleman, 2),
(1810, (11, 51), iAmerica, 'Seattle', 4, iRifleman, 2),

(1825, (3, 37), iAmerica, 'Honolulu', 4, iManOfWar, 1),
(1825, (7, 58), iAmerica, 'Anchorage', 3, iWorker, 1),
(1825, (10, 63), iAmerica, 'Prudhoe Bay', 2, iWorker, 1),

# japanese pacific expansion
(1825, (115, 33), iJapan, 'Saipan', 1, iManOfWar, 2),
(1825, (122, 32), iJapan, 'Guam', 1, iManOfWar, 2),

# brazil amazon expansion
(1825, (33, 27), iBrazil, 'Manaus', 1, iWorker, 1),
(1825, (35, 21), iBrazil, 'Goiania', 1, iWorker, 1),
(1825, (32, 24), iBrazil, 'Porto Velho', 1, iWorker, 1),

)

# do some research on dates here
tMinorStates = (
	(633, 1400, (96, 43), [iArcher, iSwordsman]),	# Tibet
	(-75, 600, (89, 46), [iHorseman]),		# Kashgar early
	(600, 1600, (89, 46), [iHorseArcher]),		# Kashgar late
	(-75, 600, (85, 47), [iHorseman]),		# Samarkand early
	(600, 1600, (85, 47), [iHorseArcher]),		# Samarkand late
	(-300, 600, (91, 31), [iArcher, iSwordsman, iWarElephant]), # Chola
	(-300, 600, (92, 33), [iArcher, iSwordsman, iWarElephant]), # Chola
	(-300, 900, (105, 49), [iHorseArcher, iSwordsman]), # Jurchen
	(1100, 1500, (60, 44), [iPikeman, iCrossbowman]), # Rome late
	(0, 1100, (60, 44), [iSpearman, iArcher]), # Rome early
)


#adjustments for tech groups
lTechGroupNotAffected = [iNetherlands, iGermany, iAmerica, iCanada, iArgentina, iMexico, iColombia, iBrazil]
lTechGroupNorthEuro = [iEngland, iVikings]
lTechGroupCentralEuro = [iFrance, iHolyRome, iItaly]
lTechGroupSouthEuro = [iRome, iSpain, iPortugal, iGreece]
lTechGroupEastEuro = [iRussia, iPoland]
lTechGroupMenaNonArab = [iByzantium, iOttomans, iIran]
lTechGroupMenaArab = [iEgypt, iMoors, iArabia]
lTechGroupChina = [iChina, iKorea, iTibet]
lTechGroupSEAsia = [iIndonesia, iKhmer, iThailand]
lTechGroupJapan = [iJapan]
lTechGroupIndia = [iMughals, iIndia, iTamils]
lTechGroupHorde = [iMongols, iTurks]
lTechGroupSubsaharan = [iEthiopia, iMali, iCongo]
lTechGroupNewWorld = [iMaya, iInca, iAztecs]
dCrusadeHappened = [False]
@handler("techAcquired")
def micromodAdjustmentsForTech(iTech, iTeam, iPlayer):
	if not constMicromodAdjustmentsForTech:
		return
		
	enterDarkAge = False
	exitDarkAge = False
	darkAgePenalty = 50
	iCiv = civ(iPlayer)	
	iTeam = team(iPlayer)
	
	# crusade? (only France, Holy Rome, England, Italy, Vikings, as historical)
	# the target is any non north/south/central euro nation -> Byzantium _can_ be a target.
	if constMicromodCrusadeEnabled and not scenario() == i1700AD:
		if (iTech == iCivilService or iTech == iFortification) and not dCrusadeHappened[0]:
			if iCiv in lTechGroupNorthEuro or iCiv in lTechGroupCentralEuro:
				orthodoxHolyCity = game.getHolyCity(iOrthodoxy)
				holyCityPlayer = orthodoxHolyCity.getOwner()
				if not ((holyCityPlayer in lTechGroupNorthEuro) or (holyCityPlayer in lTechGroupCentralEuro) or (holyCityPlayer in lTechGroupSouthEuro)):
					dCrusadeHappened[0] = True
					arrivalPlot = plot_((72, 37))
					jerusalemPlot = plot_((73, 38))
					team(iPlayer).declareWar(holyCityPlayer, True, WarPlanTypes.WARPLAN_LIMITED)
					iInfantry = getBestInfantry(iPlayer)
					iCounter = getBestCounter(iPlayer)
					iCavalry = getBestCavalry(iPlayer)
					iSiege = getBestSiege(iPlayer)
					if iInfantry:
						makeUnits(iPlayer, iInfantry, arrivalPlot, 5, UnitAITypes.UNITAI_ATTACK_CITY)			
					if iCounter:
						makeUnits(iPlayer, iCounter, arrivalPlot, 3, UnitAITypes.UNITAI_ATTACK_CITY)
					if iSiege:
						makeUnits(iPlayer, iSiege, arrivalPlot, 3, UnitAITypes.UNITAI_ATTACK_CITY)
					if iCavalry:
						makeUnits(iPlayer, iCavalry, arrivalPlot, 3, UnitAITypes.UNITAI_ATTACK_CITY)
			
	# north euros   (dark ages)
	if iCiv in lTechGroupNorthEuro:
		darkAgePenalty = 55				# strong dark age for tech leaders
		if iTech == iFeudalism:			# medium length dark age ended by medieval social/military tech
			enterDarkAge = True
		elif (iTech == iPatronage and not iTeam.isHasTech(iCompanies)) or (iTech == iCompanies and not iTeam.isHasTech(iPatronage)):
			exitDarkAge = True
	# central euros (dark ages + decentralization)
	elif iCiv in lTechGroupCentralEuro:
		darkAgePenalty = 65				# very strong dark age for uncentralized tech leaders
		if iTech == iFeudalism:			# longer dark age ended by early renaissance social/military tech
			enterDarkAge = True			
		elif (iTech == iAcademia and not iTeam.isHasTech(iFirearms)) or (iTech == iFirearms and not iTeam.isHasTech(iAcademia)):
			exitDarkAge = True
	# south+east euros (dark ages + late renaissance/early industrial economy woes)
	elif iCiv in lTechGroupSouthEuro or iCiv in lTechGroupEastEuro:
		if iTech == iFeudalism:			# classic dark age ended by late medieval social or military tech
			darkAgePenalty = 40 		
			enterDarkAge = True
		elif (iTech == iPatronage and not iTeam.isHasTech(iGunpowder)) or (iTech == iGunpowder and not iTeam.isHasTech(iPatronage)):
			darkAgePenalty = 40 		
			exitDarkAge = True
		elif iTech == iEconomics:		# short stagnation triggered by failed economics, ended by an early industrial economy/military tech
			darkAgePenalty = 25 		
			enterDarkAge = True
		elif (iTech == iMachineTools and not iTeam.isHasTech(iChemistry)) or (iTech == iChemistry and not iTeam.isHasTech(iMachineTools)):
			darkAgePenalty = 25 		
			exitDarkAge = True			
			
	#MENA non-arab
	elif (iCiv in lTechGroupMenaNonArab) or (iCiv in lTechGroupMenaArab): 
		darkAgePenalty = 33 		
		if iTech == iScientificMethod:		# long stagnation caused by the relationship between science and religion, ended by an early industrial social/military tech
			enterDarkAge = True
		elif (iTech == iNationalism and not iTeam.isHasTech(iChemistry)) or (iTech == iChemistry and not iTeam.isHasTech(iNationalism)):
			exitDarkAge = True
	#MENA arab
	elif (iCiv in lTechGroupMenaNonArab) or (iCiv in lTechGroupMenaArab): 
		if iTech == iScientificMethod:		# long stagnation caused by the relationship between science and religion,, ended by an early industrial social/military tech
			enterDarkAge = True
		elif (iTech == iNationalism and not iTeam.isHasTech(iChemistry)) or (iTech == iChemistry and not iTeam.isHasTech(iNationalism)):
			exitDarkAge = True
			
	#chinese and SEAsia: long late renaissance early modern dark age
	elif (iCiv in lTechGroupChina) or (iCiv in lTechGroupSEAsia) :
		if iTech == iStatecraft:		# long stagnation caused by failing to control the state, ended by an early industrial social/military tech
			enterDarkAge = True
		elif (iTech == iNationalism and not iTeam.isHasTech(iChemistry)) or (iTech == iChemistry and not iTeam.isHasTech(iNationalism)):
			exitDarkAge = True	
	#japan: early modern dark age (shorter than the rest of east asia)
	elif (iCiv in lTechGroupJapan):
		darkAgePenalty = 33
		if iTech == iExploration:		# short stagnation after economic contact with the rest of the world, ended by a late renaissance social/military tech
			enterDarkAge = True
		elif (iTech == iReplaceableParts and not iTeam.isHasTech(iPhysics)) or (iTech == iPhysics and not iTeam.isHasTech(iReplaceableParts)):
			exitDarkAge = True
			
	#india and hordes: fighting the europeans between exploration and nationalism.
	elif (iCiv in lTechGroupIndia) or (iCiv in lTechGroupHorde):
		if iTech == iExploration:		# very long stagnation after contact with the rest of the world, ended by an early industrial economic/military tech
			enterDarkAge = True
		elif (iTech == iChemistry and not iTeam.isHasTech(iThermodynamics)) or (iTech == iThermodynamics and not iTeam.isHasTech(iChemistry)):
			exitDarkAge = True
			
	#subsaharan and new world : medieval to late renaissance dark age 
	elif (iCiv in lTechGroupSubsaharan) or (iCiv in lTechGroupNewWorld):
		if iTech == iExploration:		# very long stagnation after contact with the rest of the world, ended by an early industrial economic/military tech
			enterDarkAge = True
		elif (iTech == iChemistry and not iTeam.isHasTech(iThermodynamics)) or (iTech == iThermodynamics and not iTeam.isHasTech(iChemistry)):
			exitDarkAge = True
	
	# enter / exit darkage 
	# pay a flat 45% more for social and technological issues
	if enterDarkAge:
		if darkAgePenalty>=60:
			message(iPlayer, "Our civilization has descended into a deep technological and cultural dark age.", color=iLightRed)
		elif darkAgePenalty>=33:
			message(iPlayer, "Our civilization has descended into a technological and cultural dark age.", color=iLightRed)
		else:
			message(iPlayer, "Our civilization has descended into a soft technological and cultural dark age.", color=iLightRed)
		changeModifier(iPlayer, iModifierResearchCost, darkAgePenalty)
		changeModifier(iPlayer, iModifierGreatPeopleThreshold, darkAgePenalty)	
		changeModifier(iPlayer, iModifierCulture, -darkAgePenalty)	
	if exitDarkAge:
		if darkAgePenalty>=60:
			message(iPlayer, "Our civilization has recovered from a deep technological and cultural dark age.", color=iPurple)
		elif darkAgePenalty>=33:
			message(iPlayer, "Our civilization has recovered from a technological and cultural dark age.", color=iPurple)
		else:
			message(iPlayer, "Our civilization has recovered from a soft technological and cultural dark age.", color=iPurple)
		changeModifier(iPlayer, iModifierResearchCost, -darkAgePenalty)
		changeModifier(iPlayer, iModifierGreatPeopleThreshold, -darkAgePenalty)
		changeModifier(iPlayer, iModifierCulture, darkAgePenalty)	
	
	# extra attack army for the US, on railroad discovery. Both the AI and the human player get this.
	if iTech == iRailroad and iCiv == iAmerica:
		spawnPlot1 = plot_((14, 44))
		spawnPlot2 = plot_((19, 42))
		spawnPlot3 = plot_((10, 48))
		makeUnits(iPlayer, iRifleman, spawnPlot1, 3, UnitAITypes.UNITAI_ATTACK_CITY)			
		makeUnits(iPlayer, iRifleman, spawnPlot2, 3, UnitAITypes.UNITAI_ATTACK_CITY)
		makeUnits(iPlayer, iRifleman, spawnPlot3, 3, UnitAITypes.UNITAI_ATTACK_CITY)
		iCounter = getBestCounter(iPlayer)
		iCavalry = getBestCavalry(iPlayer)
		iSiege = getBestSiege(iPlayer)
		if iCounter:
			makeUnits(iPlayer, iCounter, spawnPlot1, 3, UnitAITypes.UNITAI_ATTACK_CITY)
			makeUnits(iPlayer, iCounter, spawnPlot2, 3, UnitAITypes.UNITAI_ATTACK_CITY)
			makeUnits(iPlayer, iCounter, spawnPlot3, 3, UnitAITypes.UNITAI_ATTACK_CITY)
		if iSiege:
			makeUnits(iPlayer, iSiege, spawnPlot1, 2, UnitAITypes.UNITAI_ATTACK_CITY)
			makeUnits(iPlayer, iSiege, spawnPlot2, 2, UnitAITypes.UNITAI_ATTACK_CITY)
			makeUnits(iPlayer, iSiege, spawnPlot3, 2, UnitAITypes.UNITAI_ATTACK_CITY)
		if iCavalry:
			makeUnits(iPlayer, iCavalry, spawnPlot1, 2, UnitAITypes.UNITAI_ATTACK_CITY)
			makeUnits(iPlayer, iCavalry, spawnPlot2, 2, UnitAITypes.UNITAI_ATTACK_CITY)
			makeUnits(iPlayer, iCavalry, spawnPlot3, 2, UnitAITypes.UNITAI_ATTACK_CITY)

# limit colony growth
@handler("BeginGameTurn")
def micromodAdjustmentsForColonySizePerTurn(iGameTurn):
	if not constMicromodAdjustmentsForColonies:
		return

	# adjust colonies for colonial players.
	for iPlayer in players.all():
		iCiv = civ(iPlayer)	
		if not player(iCiv).isAlive():
			continue
		if iCiv == iPortugal or iCiv == iSpain or iCiv == iFrance or iCiv == iNetherlands or iCiv == iHolyRome or iCiv == iPoland or iCiv == iRome or iCiv == iByzantium or iCiv == iGermany or iCiv == iEngland or iCiv == iVikings or iCiv == iRussia or iCiv == iItaly:
			
			# flight disables limitations
			iTeam = team(iPlayer)
			if iTeam.isHasTech(iFlight):
				continue
				
			# get colonization size
			colonySizeFromTech = 2
			if iTeam.isHasTech(iFirearms):		# 2 + 1
				colonySizeFromTech += 1
			if iTeam.isHasTech(iLogistics):		# 2 + 2
				colonySizeFromTech += 1			
			if iTeam.isHasTech(iExploration):	# 2 + 3
				colonySizeFromTech += 1	
			if iTeam.isHasTech(iStatecraft):	# 2 + 4
				colonySizeFromTech += 1
			if iTeam.isHasTech(iUrbanPlanning):	# 2 + 5
				colonySizeFromTech += 1
			if iTeam.isHasTech(iEconomics):		# 2 + 6
				colonySizeFromTech += 1			
			if iTeam.isHasTech(iGeography):		# 2 + 7
				colonySizeFromTech += 1
			if iTeam.isHasTech(iCivilLiberties):# 2 + 8
				colonySizeFromTech += 1
			if iTeam.isHasTech(iSocialContract):# 2 + 9
				colonySizeFromTech += 1	
			if iTeam.isHasTech(iRepresentation):# 2 + 10
				colonySizeFromTech += 1	
			if iTeam.isHasTech(iCivilRights):	# 2 + 11
				colonySizeFromTech += 1
			
			# colony size climate penalty
			colonySizeClimatePenalty = 0
			if not iTeam.isHasTech(iScientificMethod):	# 1
				colonySizeClimatePenalty += 1		
			if not iTeam.isHasTech(iHorticulture):		# 2
				colonySizeClimatePenalty += 1		
			if not iTeam.isHasTech(iBiology):			# 3
				colonySizeClimatePenalty += 1
			if not iTeam.isHasTech(iRefrigeration):		# 4
				colonySizeClimatePenalty += 1		
			if not iTeam.isHasTech(iMicrobiology):		# 5
				colonySizeClimatePenalty += 1				
		
			for city in cities.owner(iPlayer):
				micromodAdjustColonySize(city, colonySizeFromTech, colonySizeClimatePenalty)

def micromodAdjustColonySize(city, colonySizeFromTech, colonySizeClimatePenalty):
	# exclude europe and its periphery + parts of MENA, colonizers are considered to directly control this area.
	x = city.getX()
	y = city.getY()
	if (x>=44 and x<=81 and y>=34 and y<=65):
		return
	
	# skip if under the minimum size
	if city.getPopulation() <= 2:
		return
	
	# 1. emigration push (capacity and willingness of civilians to colonize)
	maxColonySize = colonySizeFromTech
	
	# 2. development before and after colonization 
	numImprovedPlots = city.countNumImprovedPlots()
	if numImprovedPlots > 2:
		maxColonySize += 1
	if numImprovedPlots > 4:
		maxColonySize += 1
	if numImprovedPlots > 6:
		maxColonySize += 1		
	# also include the most populated and developed parts of asia (china, japan, indochina, indonesia, india, deccan, iran, south central asia)
	if (x>=70 and x<=116 and y>=40 and y<=52):		
		maxColonySize += 5
	
	# 3. social backyard areas
	# far from europe's main entry points into major continents, these areas can support large but completely ahistorical colonies: us/canada west and midwest, north and west australia
	if (x>=0 and x<=21 and y>=40 and y<=65 ) or (x>=103 and x<=117 and y>=13 and y<=22):
		maxColonySize = maxColonySize / 2 + 1
	
	# 4. climate for immigrant dominated colonies
	# In areas in which euro immigrants made up a large part of the population the colony size is limited by the player's tech, which controls how well the immigrants can handle the environment
	# Central America, Carribean, Colombia, Amazon, West Africa, Oceania
	if (x>=16 and x<=43 and y>=19 and y<=39) or (x>=46 and x<=76 and y>=15 and y<=29) or (x>=0 and x<=21 and y>=16 and y<=30):		
		# exceptions for large pop centers like Mexico, Brazil core, Bogota and Lima that supported large non-native populations
		isException = False
		if (x>=16 and x<=19 and y>=35 and y<=39) or (x>=37 and x<=42 and y>=14 and y<=19) or (x>=26 and x<=28 and y>=28 and y<=30) or (x>=25 and x<=27 and y>=21 and y<=23):
			maxColonySize += 3
			isException = True		
		if not isException:
			maxColonySize -= colonySizeClimatePenalty
		
	# clamp if necessary
	if maxColonySize < 2:
		maxColonySize = 2
	if city.getPopulation() > maxColonySize:
		city.setPopulation(maxColonySize)

		
#handicap level modifier
iHandicapOld = (game.getHandicapType() - 1)

		
@handler("BeginGameTurn")
def helpMinorStates():
	if every(20):
		for iStartYear, iEndYear, tPlot, lUnits in tMinorStates:
			if year().between(iStartYear, iEndYear):
				plot = plot_(tPlot)
				iOwner = plot.getOwner()
				if plot.isCity() and is_minor(iOwner) and plot.getNumUnits() < 4:
					makeUnit(iOwner, random_entry(lUnits), plot)


@handler("BeginGameTurn")
def spawnBarbarians(iGameTurn):
	iHandicap = infos.handicap().getBarbarianSpawnModifier()

	if year().between(-3000, -850):
		if iHandicap >= 0:
			checkSpawn(iBarbarian, iWarrior, 1, (76, 46), (99, 53), spawnMinors, iGameTurn, 5, 0)
		
		checkSpawn(iBarbarian, iWolf, 1, (75, 54), (104, 64), spawnNatives, iGameTurn, 5, 2)
		checkSpawn(iBarbarian, iBear, 1, (75, 54), (104, 64), spawnNatives, iGameTurn, 5, 4)
		checkLimitedSpawn(iBarbarian, iLion, 1, 5, (60, 10), (72, 28), spawnNatives, iGameTurn, 5, 1)
		checkLimitedSpawn(iBarbarian, iPanther, 1, 5, (60, 10), (72, 28), spawnNatives, iGameTurn, 5, 3)
		
	#celts
	if year().between(-650, -110):
		checkSpawn(iCelts, iGallicWarrior, 1, (49, 46), (65, 52), spawnMinors, iGameTurn, 6, 0)
		if iHandicap >= 0:
			checkSpawn(iCelts, iAxeman, 1, (49, 46), (65, 52), spawnMinors, iGameTurn, 8, 5, ["TXT_KEY_ADJECTIVE_GAUL"])

	#norse
	if year().between(-650, 550):
		checkSpawn(iBarbarian, iGalley, 1, (50, 49), (61, 55), spawnPirates, iGameTurn, 20, 0, ["TXT_KEY_ADJECTIVE_NORSE"])
		
	#mongolia
	if year().between(-210, 400):
		checkSpawn(iBarbarian, iHorseArcher, 3 + iHandicap, (94, 48), (107, 54), spawnNomads, iGameTurn, 7-iHandicap, 0, ["TXT_KEY_ADJECTIVE_XIONGNU"])
	elif year().between(400, 900):
		iNumUnits = 3 + iHandicap
		checkSpawn(iBarbarian, iHorseArcher, iNumUnits, (91, 50), (107, 54), spawnNomads, iGameTurn, 6-iHandicap, 0, ["TXT_KEY_ADJECTIVE_GOKTURK", "TXT_KEY_ADJECTIVE_UIGHUR"])
	elif year().between(900, 1100):
		iNumUnits = 3 + iHandicap
		checkSpawn(iBarbarian, iKeshik, iNumUnits, (94, 48), (107, 54), spawnInvaders, iGameTurn, 6, 0, ["TXT_KEY_ADJECTIVE_JURCHEN", "TXT_KEY_ADJECTIVE_KHITAN"])
		
	#tibet
	if year().between(-350, 200):
		checkSpawn(iBarbarian, iLightSwordsman, 1 + iHandicap, (92, 41), (99, 45), spawnMinors, iGameTurn, 10-iHandicap, 3, ["TXT_KEY_ADJECTIVE_TIBETAN"])
	elif year().between(200, 1100):
		checkSpawn(iBarbarian, iSwordsman, 1 + iHandicap, (92, 41), (99, 45), spawnMinors, iGameTurn, 10-iHandicap, 3, ["TXT_KEY_ADJECTIVE_TIBETAN"])

	# Deccan barbarians
	if year().between(-1000, 1200):
		iUnit = iArcher
		iStrength = iHandicap
		if iGameTurn >= year(-500): iUnit = iAxeman
		if iGameTurn >= year(0): iStrength += 1
		if iGameTurn >= year(200): iUnit = iSwordsman
		
		checkSpawn(iBarbarian, iUnit, iStrength, (87, 23), (96, 37), spawnInvaders, iGameTurn, 8-iHandicap, 0, ["Hindi"])
		
	# elephants in india pre-khmer
	if year().between(-210, 700):
		checkSpawn(iBarbarian, iWarElephant, 1, (86, 31), (100, 41), spawnInvaders, iGameTurn, 8-iHandicap, 4)

	#Indo-Scythians
	if year().between(-200, 400):
		checkSpawn(iBarbarian, iHorseman, 2, (84, 40), (89, 43), spawnNomads, iGameTurn, 8-iHandicap, 4, ["TXT_KEY_ADJECTIVE_INDO_SCYTHIAN"])

	#Kushana
	if year().between(30, 220):
		checkSpawn(iBarbarian, iAsvaka, 3+iHandicap, (84, 40), (89, 43), spawnInvaders, iGameTurn, 8, 3, ["TXT_KEY_ADJECTIVE_KUSHANA"])

	#Hephtalites
	if year().between(400, 550):
		checkSpawn(iBarbarian, iHorseArcher, 2+iHandicap, (84, 40), (89, 43), spawnInvaders, iGameTurn, 5-iHandicap, 2, ["TXT_KEY_ADJECTIVE_HEPHTHALITE"])

	# Holkans in classical Mesoamerica
	if year().between(-200, 100):
		checkSpawn(iBarbarian, iHolkan, 1, (17, 31), (25, 37), spawnUprising, iGameTurn, 7, 5)
	if year().between(100, 600):
		checkSpawn(iBarbarian, iHolkan, 1, (17, 31), (25, 37), spawnUprising, iGameTurn, 6, 4)	
	elif year().between(600, 1000):
		checkSpawn(iBarbarian, iHolkan, 1, (17, 31), (25, 37), spawnUprising, iGameTurn, 4, 2)
		
	#pirates in Mediterranean
	if year().between(-210, 50):
		checkSpawn(iBarbarian, iWarGalley, 1, (49, 37), (72, 44), spawnPirates, iGameTurn, 8, 0)
	#pirates in Barbary coast
	if not player(iMoors).isAlive():
		if year().between(-50, 700):
			checkSpawn(iBarbarian, iWarGalley, 1, (46, 30), (62, 39), spawnPirates, iGameTurn, 18, 0)
		elif year().between(700, 1400):
			checkSpawn(iBarbarian, iWarGalley, 1, (46, 30), (62, 39), spawnPirates, iGameTurn, 8, 0)
	#pirates in Indian ocean
	if year().between(-650, 700):
		checkSpawn(iBarbarian, iWarGalley, 1, (72, 20), (91, 36), spawnPirates, iGameTurn, 18, 0)
	elif year().between(700, 1700):
		checkSpawn(iBarbarian, iHeavyGalley, 1, (72, 20), (91, 36), spawnPirates, iGameTurn, 10, 0)

	# Leoreth: Barbarians in Anatolia (Hittites), replace Hattusas spawn
	if year().between(-2000, -800):
		checkSpawn(iBarbarian, iHuluganni, 1 + iHandicap, (68, 42), (74, 45), spawnInvaders, iGameTurn, 16, 0, ["TXT_KEY_ADJECTIVE_HITTITE"])

	#barbarians in europe
	if year().between(-210, 470):
		checkSpawn(iBarbarian, iAxeman, 3 + iHandicap, (50, 45), (63, 52), spawnInvaders, iGameTurn, 10, 0, ["TXT_KEY_ADJECTIVE_GERMANIC"])
		checkSpawn(iBarbarian, iAxeman, 2 + iHandicap, (64, 49), (69, 55), spawnInvaders, iGameTurn, 12, 2, ["TXT_KEY_ADJECTIVE_GERMANIC"])
	# Leoreth: begins 100 AD instead of 50 AD
	if year().between(100, 470):
		checkSpawn(iBarbarian, iSwordsman, 3, (58, 45), (70, 55), spawnInvaders, iGameTurn, 8, 5, ["TXT_KEY_ADJECTIVE_GERMANIC"])
	if year().between(300, 550):
		checkSpawn(iBarbarian, iAxeman, 4 + iHandicap, (49, 41), (56, 52), spawnInvaders, iGameTurn, 5, 4, ["TXT_KEY_ADJECTIVE_VISIGOTHIC"])
		checkSpawn(iBarbarian, iSwordsman, 4 + iHandicap, (49, 41), (57, 52), spawnInvaders, iGameTurn, 5, 2, ["TXT_KEY_ADJECTIVE_VISIGOTHIC"])
		checkSpawn(iBarbarian, iHorseArcher, 3, (55, 49), (65, 53), spawnInvaders, iGameTurn, 5, 0, ["TXT_KEY_ADJECTIVE_HUNNIC"])
	if year().between(300, 700):
		checkSpawn(iBarbarian, iHorseArcher, 3 + iHandicap, (58, 50), (88, 53), spawnInvaders, iGameTurn, 3, 2, ["TXT_KEY_ADJECTIVE_HUNNIC"])

	#Leoreth: barbarians in Balkans / Black Sea until the High Middle Ages (Bulgarians, Cumans, Pechenegs)
	if year().between(680, 1000):
		checkSpawn(iBarbarian, iHorseArcher, 3 + iHandicap, (64, 45), (69, 49), spawnInvaders, iGameTurn, 6, 2, ["TXT_KEY_ADJECTIVE_AVAR", "TXT_KEY_ADJECTIVE_BULGAR"])
	if year().between(900, 1200):
		checkSpawn(iBarbarian, iHorseArcher, 3 + iHandicap, (68, 48), (78, 50), spawnInvaders, iGameTurn, 8, 5, ["TXT_KEY_ADJECTIVE_CUMAN"])
		
	#barbarians in central asia
	if year().between(-1600, -850):
		checkLimitedSpawn(iBarbarian, iVulture, 1, 3, (74, 34), (78, 44), spawnNomads, iGameTurn, 8-iHandicap, 2, ["TXT_KEY_ADJECTIVE_ASSYRIAN"])
	elif year().between(-850, -200):
		checkSpawn(iBarbarian, iHorseman, 1 + iHandicap, (79, 41), (84, 49), spawnInvaders, iGameTurn, 10-2*iHandicap, 3, ["TXT_KEY_ADJECTIVE_SCYTHIAN"])
	elif year().between(-200, 600):
		checkSpawn(iBarbarian, iHorseman, 2 + iHandicap, (79, 41), (84, 49), spawnInvaders, iGameTurn, 7-iHandicap, 2, ["TXT_KEY_ADJECTIVE_PARTHIAN"])
	elif year().between(600, 900):
		checkSpawn(iBarbarian, iOghuz, 2 + iHandicap, (78, 42), (88, 50), spawnNomads, iGameTurn, 8-iHandicap, 2)
	elif year().between(900, 1040):
		checkSpawn(iBarbarian, iOghuz, 2 + iHandicap, (78, 42), (90, 52), spawnNomads, iGameTurn, 6-iHandicap, 2)

	# late Central Asian barbarians
	if year().between(1200, 1600):
		if not cities.start(70, 48).end(80, 59).owner(iMongols):
			checkSpawn(iBarbarian, iKeshik, 1 + iHandicap, (74, 47), (81, 47), spawnNomads, iGameTurn, 10-iHandicap, 5, ["TXT_KEY_ADJECTIVE_TATAR", "TXT_KEY_ADJECTIVE_NOGAI"])
	if year().between(1400, 1700):
		if cities.start(80, 47).end(88, 53):
			checkSpawn(iBarbarian, iKeshik, 1 + iHandicap, (80, 47), (88, 53), spawnNomads, iGameTurn, 10-2*iHandicap, 2, ["TXT_KEY_ADJECTIVE_UZBEK", "TXT_KEY_ADJECTIVE_KAZAKH"])
		
	#barbarians in Elam
	if year().between(-1600, -1000):
		checkSpawn(iBarbarian, iChariot, 1, (81, 37), (87, 45), spawnMinors, iGameTurn, 9-iHandicap, 0, ["TXT_KEY_ADJECTIVE_ELAMITE"])

	#barbarians in north africa
	if year().between(-210, 50):
		checkSpawn(iBarbarian, iCamelRider, 1, (54, 31), (67, 35), spawnNomads, iGameTurn, 9-iHandicap, 3, ["TXT_KEY_ADJECTIVE_BERBER"])
	elif year().between(50, 900):
		if scenario() == i3000BC:  #late start condition
			checkSpawn(iBarbarian, iCamelRider, 1 + iHandicap, (54, 31), (67, 35), spawnNomads, iGameTurn, 10-iHandicap, 5, ["TXT_KEY_ADJECTIVE_BERBER"])
	elif year().between(900, 1800):
		checkSpawn(iBarbarian, iCamelArcher, 1, (54, 27), (67, 35), spawnNomads, iGameTurn, 10-iHandicap, 4, ["TXT_KEY_ADJECTIVE_BERBER"])
		
	#camels in arabia
	if year().between(190, 550):
		checkSpawn(iBarbarian, iCamelArcher, 1, (73, 30), (82, 36), spawnNomads, iGameTurn, 9-iHandicap, 7, ["TXT_KEY_ADJECTIVE_BEDOUIN"])
	if year().between(-800, 1300) and includesActiveHuman(iEgypt, iArabia):
		iNumUnits = iHandicap
		if scenario() == i3000BC: iNumUnits += 1
		checkSpawn(iBarbarian, iMedjay, iNumUnits, (66, 28), (71, 34), spawnUprising, iGameTurn, 12, 4, ["TXT_KEY_ADJECTIVE_NUBIAN"])
	if year().between(450, 1900):
		if scenario() == i3000BC:
			checkSpawn(iNative, iNativeRaider, 2 + iHandicap, (60, 10), (72, 27), spawnNatives, iGameTurn, 10, 4)
		else:
			checkSpawn(iNative, iNativeRaider, 2 + iHandicap, (60, 10), (72, 27), spawnNatives, iGameTurn, 15, 4)
	elif year().between(1600, 1800):
		checkSpawn(iNative, iPombos, 2 + iHandicap, (60, 10), (72, 27), spawnNatives, iGameTurn, 10, 4)
		
	#west africa
	if year().between(450, 1800):
		checkSpawn(iNative, iNativeRaider, 2, (48, 22), (66, 29), spawnNatives, iGameTurn, 16, 10)
		
		checkSpawn(iNative, iNativeArcher, 1, (48, 22), (66, 29), spawnNatives, iGameTurn, 12, 8)
		checkSpawn(iNative, iNativeWarrior, 2, (48, 22), (66, 29), spawnNatives, iGameTurn, 12, 2)
		
	if year().between(1200, 1700):
		checkSpawn(iBarbarian, iFarari, 1, (48, 26), (65, 37), spawnMinors, iGameTurn, 16, 4, ["TXT_KEY_ADJECTIVE_SONGHAI"])

	#American natives
	if year().between(600, 1100):
		checkSpawn(iNative, iDogSoldier, 1 + iHandicap, (15, 38), (24, 47), spawnNatives, iGameTurn, 20, 0)
		if scenario() == i3000BC:  #late start condition
			checkSpawn(iNative, iJaguar, 3, (15, 38), (24, 47), spawnNatives, iGameTurn, 16 - 2*iHandicap, 10)
		else:  #late start condition
			checkSpawn(iNative, iJaguar, 2, (15, 38), (24, 47), spawnNatives, iGameTurn, 16 - 2*iHandicap, 10)
	if year().between(1300, 1600):
		checkSpawn(iNative, iDogSoldier, 2 + iHandicap, (15, 38), (24, 47), spawnNatives, iGameTurn, 8, 0)
	if year().between(1400, 1800):
		checkSpawn(iNative, iDogSoldier, 1 + iHandicap, (11, 44), (33, 51), spawnUprising, iGameTurn, 12, 0)
		checkSpawn(iNative, iDogSoldier, 1 + iHandicap, (11, 44), (33, 51), spawnUprising, iGameTurn, 12, 6)
	if year().between(1300, 1600):
		if iGameTurn % 18 == 0:
			if not plot_(27, 29).isUnit():
				makeUnits(iNative, iDogSoldier, (27, 29), 2 + iHandicap, UnitAITypes.UNITAI_ATTACK)
		elif iGameTurn % 18 == 9:
			if not plot_(30, 13).isUnit():
				makeUnits(iNative, iDogSoldier, (30, 13), 2 + iHandicap, UnitAITypes.UNITAI_ATTACK)
	
	if includesActiveHuman(iAmerica, iEngland, iFrance):
		if year().between(1700, 1900):
			checkSpawn(iNative, iMountedBrave, 1 + iHandicap, (15, 44), (24, 52), spawnNomads, iGameTurn, 12 - iHandicap, 2)
		
		if year().between(1500, 1850):
			checkSpawn(iNative, iMohawk, 1, (24, 46), (30, 51), spawnUprising, iGameTurn, 8, 4)
			
	if iGameTurn == year(-500):
		plot_(19, 35).setImprovementType(iHut)
		makeUnits(iNative, iHolkan, (19, 35), 2, UnitAITypes.UNITAI_ATTACK)
		
	# Oromos in the Horn of Africa
	if year().between(1500, 1700):
		iNumUnits = 1
		if player(iEthiopia).isAlive():
			iNumUnits += 1
			if year().between(1600, 1700): iNumUnits += 1
		checkSpawn(iBarbarian, iOromoWarrior, iNumUnits, (69, 25), (74, 28), spawnInvaders, iGameTurn, 8, 3)
			
	#pirates in the Caribbean
	if year().between(1600, 1800):
		checkSpawn(iNative, iPrivateer, 1, (24, 32), (35, 46), spawnPirates, iGameTurn, 5, 0)
	#pirates in Asia
	if year().between(1500, 1900):
		checkSpawn(iNative, iPrivateer, 1, (72, 24), (110, 36), spawnPirates, iGameTurn, 8, 0)

	if iGameTurn < year(tMinorCities[len(tMinorCities)-1][0])+10:
		foundMinorCities(iGameTurn)

	if iGameTurn == year(dBirth[iInca]):
		if player(iInca).isHuman():
			makeUnit(iNative, iAucac, (24, 26))
			makeUnit(iNative, iAucac, (25, 23))
				
def foundMinorCities(iGameTurn):
	for i, (iYear, tPlot, iCiv, sName, iPopulation, iUnitType, iNumUnits) in enumerate(tMinorCities):
		# quick skip
		if data.lMinorCityFounded[i]: continue
		if plot(tPlot).isCity(): continue
	
		# scenario hack to try to spawn cities retroactively. This might have unintended consequences.
		forceRetroactiveSpawn = False
		if scenario() == i1700AD:							
			if iGameTurn <= year(1710):							# try to retroactively spawn previous cities (many of them will fail due to other cities occupying their place)
																# after plot changes, that happen in game.getGameTurn() + 1
				if iYear < 600:	 continue						# ancient cities are skipped
				forceRetroactiveSpawn = True
				
				if sName == 'Newcastle' or sName == 'Liverpool' or sName == 'Plymouth':
					iCiv = iEngland
					iPopulation = 3
				elif sName == 'Prague' or sName == 'Venice':
					iCiv = iHolyRome
					iPopulation = 6
				elif sName == 'Ephesus':
					iCiv = iOttomans
					iPopulation = 8
					sName = 'Izmir'
				elif sName == 'Iraklion':
					iCiv = iOttomans
				elif iCiv == iByzantium or iCiv == iArabia: 		
					iCiv = iIndependent
					# as arab and greek revolts to give the ottomans something to watch out for
				elif sName == 'Brest' or sName == 'Lyon':
					iCiv = iFrance
					iPopulation = 9		# populated france
				elif sName == 'Sarai':
					iCiv = iRussia
					sName = 'Tsaritsyn'
					# let Russia have its time of troubles with the other barb/independent spawn
				#let china fight the independents
			else:
				if iGameTurn > year(iYear)+10: continue
			if iGameTurn < year(iYear) : continue		
		else:
			if iGameTurn < year(iYear): continue		
			if iGameTurn > year(iYear)+10: continue
		
		# major civ is dead?
		if iCiv != iNative and iCiv !=iIndependent and iCiv != iIndependent2 and iCiv !=iBarbarian:
			if not player(iCiv).isAlive(): continue
		
		# only colonize for the human player if constMicromodColonizeForHumanPlayer or if colonizing retroactively for the 1700 scenario
		if player(iCiv).isHuman() and not constMicromodColonizeForHumanPlayer and not forceRetroactiveSpawn: continue
		
		# expand with religion
		lReligions = []
		if sName == 'Kyiv': lReligions = [iOrthodoxy]
		elif iCiv == iCelts and scenario() != i3000BC: iCiv = civ(players.independent().random())
		elif sName == 'Buda': lReligions = [iCatholicism, iOrthodoxy]
		elif sName == 'Milano': lReligions = [iCatholicism]
		elif iCiv == iJapan: lReligions = [iBuddhism]
		elif iCiv == iByzantium: lReligions = [iOrthodoxy]
		elif iCiv == iOttomans or iCiv == iArabia or iCiv == iMoors or iCiv == iIran: lReligions = [iIslam]
		elif iCiv == iRussia: lReligions = [iOrthodoxy]		
		elif iCiv == iEngland or iCiv == iAmerica or iCiv == iNetherlands or iCiv == iGermany:	# 75% protestant, 25% catholic&protestant
			if rand(100) < 75:
				lReligions = [iProtestantism]
			else:
				lReligions = [iProtestantism, iCatholicism]
		elif iCiv == iFrance or iCiv == iSpain or iCiv == iPortugal:	# 75% catholic, 25% catholic&protestant
			if rand(100) < 75:
				lReligions = [iCatholicism]
			else:
				lReligions = [iCatholicism, iCatholicism]
		
		if not isFree(iCiv, tPlot, bNoCity=True, bNoEnemyUnit=False, bCanEnter=False, bNoCulture=False): continue
		
		evacuate(slot(iCiv), tPlot)
	
		if foundCity(iCiv, tPlot, sName, iPopulation, iUnitType, iNumUnits, lReligions):
			data.lMinorCityFounded[i] = True
	
def foundCity(iCiv, (x, y), sName, iPopulation, iUnitType = -1, iNumUnits = -1, lReligions = []):
	iPlayer = slot(iCiv)
	pPlayer = player(iPlayer)
	plot(x, y).setOwner(iPlayer)
	pPlayer.found(x, y)
	
	founded = city(x, y)
	if founded:
		founded.setName(sName, False)
		founded.setPopulation(iPopulation)
		
		# culture only for independents
		if iCiv != iNative and iCiv != iBarbarian:
			plot(founded).changeCulture(iPlayer, 10 * (game.getCurrentEra() + 1), True)
			founded.changeCulture(iPlayer, 10 * (game.getCurrentEra() + 1), True)
		else:
			plot(founded).changeCulture(iPlayer, 0, True)
			founded.changeCulture(iPlayer, 0, True)
		iUnitTypeDefend = getBestDefender(iPlayer)
		makeUnits(iPlayer, iUnitTypeDefend, founded, 1)	#more defenders
		
		if iNumUnits > 0 and iUnitType > 0:
			makeUnits(iPlayer, iUnitType, founded, iNumUnits)
			
		for iReligion in lReligions:
			if game.isReligionFounded(iReligion):
				founded.setHasReligion(iReligion, True, False, False)
				
		return True
	
	return False

def checkLimitedSpawn(iCiv, iUnitType, iNumUnits, iMaxUnits, tTL, tBR, spawnFunction, iTurn, iPeriod, iRest, lAdj=[]):
	iAreaUnits = plots.start(tTL).end(tBR).units().owner(iCiv).count()
	if iAreaUnits < iMaxUnits:
		checkSpawn(iCiv, iUnitType, iNumUnits, tTL, tBR, spawnFunction, iTurn, iPeriod, iRest, lAdj)
					
# Leoreth: new ways to spawn barbarians
def checkSpawn(iCiv, iUnitType, iNumUnits, tTL, tBR, spawnFunction, iTurn, iPeriod, iRest, lAdj=[]):	
	if periodic(iPeriod):
		spawnFunction(slot(iCiv), iUnitType, iNumUnits, tTL, tBR, random_entry(lAdj))
			
def possibleTiles(tTL, tBR, bWater=False, bTerritory=False, bBorder=False, bImpassable=False, bNearCity=False):
	return plots.start(tTL).end(tBR).where(lambda p: possibleTile(p, bWater, bTerritory, bBorder, bImpassable, bNearCity))
	
def possibleTile(plot, bWater, bTerritory, bBorder, bImpassable, bNearCity):
	# never on peaks
	if plot.isPeak(): return False
	
	# only land or water
	if bWater != plot.isWater(): return False
	
	# only inside territory if specified
	if not bTerritory and plot.isOwned(): return False
	
	# never directly next to cities
	if cities.surrounding(plot): return False
	
	# never on tiles with units
	if plot.isUnit(): return False
	
	# never in marsh (impassable)
	if plot.getFeatureType() == iMarsh: return False
	
	# allow other impassable terrain (ocean, jungle)
	if not bImpassable:
		if plot.getTerrainType() == iOcean: return False
		if plot.getFeatureType() == iJungle: return False
	
	# restrict to borders if specified
	if bBorder and not plots.surrounding(plot).notowner(plot.getOwner()): return False
	
	# near a city if specified (next to cities excluded above)
	if bNearCity and not plots.surrounding(plot, radius=2).where(lambda p: not p.isCity()): return False
	
	# not on landmasses without cities
	if not bWater and map.getArea(plot.getArea()).getNumCities() == 0: return False
	
	return True

def spawnPirates(iPlayer, iUnitType, iNumUnits, tTL, tBR, sAdj=""):
	'''Leoreth: spawns all ships at the same coastal spot, out to pillage and disrupt trade, can spawn inside borders'''
	plot = possibleTiles(tTL, tBR, bWater=True, bTerritory=False).random()
	
	if plot:
		makeUnits(iPlayer, iUnitType, plot, iNumUnits, UnitAITypes.UNITAI_PIRATE_SEA).adjective(sAdj)
	
def spawnNatives(iPlayer, iUnitType, iNumUnits, tTL, tBR, sAdj=""):
	'''Leoreth: outside of territory, in jungles, all dispersed on several plots, out to pillage'''
	for plot in possibleTiles(tTL, tBR, bTerritory=False, bImpassable=True).sample(iNumUnits):
		makeUnits(iPlayer, iUnitType, plot, 1, UnitAITypes.UNITAI_ATTACK).adjective(sAdj)
		
def spawnMinors(iPlayer, iUnitType, iNumUnits, tTL, tBR, sAdj=""):
	'''Leoreth: represents minor states without ingame cities
			outside of territory, not in jungles, in groups, passive'''
	plot = possibleTiles(tTL, tBR, bTerritory=False).random()
	
	if plot:
		makeUnits(iPlayer, iUnitType, plot, iNumUnits, UnitAITypes.UNITAI_ATTACK).adjective(sAdj)
	
def spawnNomads(iPlayer, iUnitType, iNumUnits, tTL, tBR, sAdj=""):
	'''Leoreth: represents aggressive steppe nomads etc.
			outside of territory, not in jungles, in small groups, target cities'''
	plot = possibleTiles(tTL, tBR, bTerritory=False).random()
	
	if plot:
		makeUnits(iPlayer, iUnitType, plot, iNumUnits, UnitAITypes.UNITAI_ATTACK).adjective(sAdj)
		
def spawnInvaders(iPlayer, iUnitType, iNumUnits, tTL, tBR, sAdj=""):
	'''Leoreth: represents large invasion forces and migration movements
			inside of territory, not in jungles, in groups, target cities'''
	plot = possibleTiles(tTL, tBR, bTerritory=True, bBorder=True).random()
	
	if plot:
		makeUnits(iPlayer, iUnitType, plot, iNumUnits, UnitAITypes.UNITAI_ATTACK).adjective(sAdj)
		
def spawnUprising(iPlayer, iUnitType, iNumUnits, tTL, tBR, sAdj=""):
	''' Leoreth: represents uprisings of Natives against colonial settlements, especially North America
			 spawns units in a free plot in the second ring of a random target city in the area
			 (also used for units from warring city states in classical Mesoamerica)'''
	plot = possibleTiles(tTL, tBR, bTerritory=True, bNearCity=True).random()
	
	if plot:
		makeUnits(iPlayer, iUnitType, plot, iNumUnits, UnitAITypes.UNITAI_ATTACK).adjective(sAdj)
		
def includesActiveHuman(*civs):
	return civ() in civs and year(dBirth[active()]) <= year()