# Rhye's and Fall of Civilization - Dynamic resources

from CvPythonExtensions import *
import CvUtil
import PyHelpers
from Core import *
from RFCUtils import * # edead
from StoredData import data

from Events import handler


@handler("GameStart")
def setupOnGameStart():
	setup()
	
@handler("OnLoad")
def setupOnLoad():
	setup()

def setup():
	global dResources
	dResources = TileDict(dResourcesDict, year)
	
	global dSpawnResources
	dSpawnResources = TileDict(dSpawnResourcesDict)
	
	global dRemovedResources
	dRemovedResources = TileDict(dRemovedResourcesDict, year)
	
	global dPlotTypes
	dPlotTypes = TileDict(dPlotTypesDict, year)
	
	global dFeatures
	dFeatures = TileDict(dFeaturesDict, year)
	
	for tile in lNewfoundlandCapes:
		dFeatures[tile] = (700, iCape)
	
	global dRemovedFeatures
	dRemovedFeatures = TileDict(dRemovedFeaturesDict, year)
	
	for tile in lNewfoundlandCapes:
		dRemovedFeatures[tile] = 1500
		
	global dRoutes
	dRoutes = appenddict(dict((year(iYear), tiles) for iYear, tiles in dRoutesDict.items()))


### Constants ###

# initialise bonuses variables

lSilkRoute = [(85,48), (86,49), (87,48), (88,47), (89,46), (90,47), (90,45), (91,47), (91,45), (92,48), (93,48), (93,46), (94,47), (95,47), (96,47), (97,47), (98,47), (99,46)]
lNewfoundlandCapes = [(34, 52), (34, 53), (34, 54), (35, 52), (36, 52), (35, 55), (35, 56), (35, 57), (36, 51), (36, 58), (36, 59)]


constPlotChangeDate = -3000	# plot changes date (needs to preceed constOldWorldDate)
constOldWorldDate = -2950	# old world resource date
constLateAncientDate = 500	# late ancient
constColonizerDate = 1550	# early columbian exchange

# old world resources
dResourcesDict = {
	(88, 37)  : (constOldWorldDate, iHorse),   # Gujarat
	(78, 42)  : (constOldWorldDate,  iCopper),  # Assyria
	(88, 47)  : (constOldWorldDate,  iSilk),    # Silk Route
	(85, 46)  : (constOldWorldDate,  iSilk),    # Silk Route
	(108, 47) : (constOldWorldDate,   iPig),     # Hanseong
	(71, 34)  : (constOldWorldDate,   iIron),    # Egypt
	(72, 24)  : (constOldWorldDate,  iSugar),   # East Africa
	(70, 17)  : (constOldWorldDate,  iSugar),   # Zimbabwe
	(67, 11)  : (constOldWorldDate,  iSugar),   # South Africa
	(66, 23)  : (constOldWorldDate,  iBanana),  # Central Africa
	(64, 20)  : (constOldWorldDate,  iBanana),  # Central Africa
	(57, 46)  : (constOldWorldDate,  iWine),    # Savoy
	(57, 45)  : (constOldWorldDate,  iClam),    # Savoy
	(50, 44)  : (constOldWorldDate,  iIron),    # Portugal
	(96, 36)  : (constOldWorldDate,  iFish),    # Bengal
	(56, 54)  : (constOldWorldDate,  iFish),    # Amsterdam
	(57, 52)  : (constOldWorldDate,  iWheat),   # Amsterdam
	(58, 52)  : (constOldWorldDate,  iCow),     # Amsterdam
	(29, 52)  : (constColonizerDate,  iCow),     # Montreal
	(18, 53)  : (constColonizerDate,  iCow),     # Alberta
	(12, 52)  : (constColonizerDate,  iCow),     # British Columbia
	(28, 46)  : (constColonizerDate,  iCow),     # Washington area
	(30, 49)  : (constColonizerDate,  iCow),     # New York area
	(22, 43)  : (constColonizerDate,  iCow),     # Louisiana
	(18, 46)  : (constColonizerDate,  iCow),     # Colorado
	(20, 45)  : (constColonizerDate,  iCow),     # Texas
	(37, 14)  : (constColonizerDate,  iCow),     # Argentina
	(33, 11)  : (constColonizerDate,  iCow),     # Argentina
	(35, 10)  : (constColonizerDate,  iCow),     # Pampas
	(24, 43)  : (constColonizerDate,  iCotton),  # near Florida
	(23, 45)  : (constColonizerDate,  iCotton),  # Louisiana
	(22, 44)  : (constColonizerDate,  iCotton),  # Louisiana
	(13, 45)  : (constColonizerDate,  iCotton),  # California
	(25, 49)  : (constColonizerDate,  iPig),     # Lakes
	(19, 51)  : (constColonizerDate,  iSheep),   # Canadian border
	(19, 48)  : (constColonizerDate,  iWheat),   # Midwest
	(20, 53)  : (constColonizerDate,  iWheat),   # Manitoba
	(22, 33)  : (constColonizerDate,  iBanana),  # Guatemala
	(27, 31)  : (constColonizerDate,  iBanana),  # Colombia
	(43, 23)  : (constColonizerDate,  iBanana),  # Brazil
	(39, 26)  : (constColonizerDate,  iBanana),  # Brazil
	(49, 44)  : (constColonizerDate,  iCorn),    # Galicia
	(54, 48)  : (constColonizerDate,  iCorn),    # France
	(67, 47)  : (constColonizerDate,  iCorn),    # Romania
	(106, 50) : (constColonizerDate,  iCorn),    # Manchuria
	(77, 52)  : (constColonizerDate,  iCorn),    # Caricyn
	(92, 35)  : (constColonizerDate,  iSpices),  # Deccan
	(16, 54)  : (constColonizerDate,  iHorse),   # Alberta
	(26, 45)  : (constColonizerDate,  iHorse),   # Washington area
	(21, 48)  : (constColonizerDate,  iHorse),   # Midwest
	(19, 45)  : (constColonizerDate,  iHorse),   # Texas
	(17, 42)  : (constColonizerDate,  iHorse),   # Mexico
	(40, 25)  : (constColonizerDate,  iHorse),   # Brazil
	(33, 10)  : (constColonizerDate,  iHorse),   # Buenos Aires area
	(32, 8)   : (constColonizerDate,  iHorse),   # Pampas
	(30, 30)  : (constColonizerDate,  iHorse),   # Venezuela
	(27, 36)  : (constColonizerDate,  iSugar),   # Caribbean
	(39, 25)  : (constColonizerDate,  iSugar),   # Brazil
	(37, 20)  : (constColonizerDate,  iSugar),   # inner Brazil
	(29, 37)  : (constColonizerDate,  iSugar),   # Hispaniola
	(104, 52) : (constColonizerDate,  iCorn),    # Manchuria
	(89, 36)  : (constColonizerDate,  iCorn),    # India
	(38, 18)  : (constColonizerDate,  iCoffee),  # Brazil
	(39, 20)  : (constColonizerDate,  iCoffee),  # Brazil
	(38, 22)  : (constColonizerDate,  iCoffee),  # Brazil
	(27, 30)  : (constColonizerDate,  iCoffee),  # Colombia
	(29, 30)  : (constColonizerDate,  iCoffee),  # Colombia
	(26, 27)  : (constColonizerDate,  iCoffee),  # Colombia
	(104, 25) : (constColonizerDate,  iCoffee),  # Java
	(67, 44)  : (constColonizerDate,  iTobacco), # Turkey
	(39, 16)  : (constOldWorldDate,  iFish),    # Brazil
	(70, 59)  : (constOldWorldDate,  iDeer),    # St Petersburg
	(12, 45)  : (constColonizerDate,  iWine),    # California
	(31, 10)  : (constColonizerDate,  iWine),    # Andes
	(113, 11) : (constColonizerDate,  iWine),    # Barossa Valley
	(114, 11) : (constColonizerDate,  iSheep),   # Australia
	(116, 13) : (constColonizerDate,  iSheep),   # Australia
	(121, 5)  : (constColonizerDate,  iSheep),   # New Zealand
	(58, 47)  : (constColonizerDate,  iRice),    # Vercelli
	(11, 49)  : (constColonizerDate,  iRice),    # California
	(11, 45)  : (constOldWorldDate,  iFish),    # California
	(87, 35)  : (constOldWorldDate,  iFish),    # Mumbai
	(115, 52) : (constOldWorldDate,  iCow),     # Hokkaido
	(1, 38)   : (constOldWorldDate,  iSugar),   # Hawaii
	(5, 36)   : (constOldWorldDate,  iBanana),  # Hawaii
	(108, 18) : (constOldWorldDate,  iCamel),   # Australia
	# conditionals
	(90, 28) : (constOldWorldDate, iFish),
	(95, 43) : (constOldWorldDate, iWheat),
	(97, 44) : (constOldWorldDate, iHorse),
	(78, 51) : (constOldWorldDate, iSilk),
	(61, 22) : (constOldWorldDate, iCotton),
	(63, 19) : (constOldWorldDate, iIvory),
	(61, 24) : (constOldWorldDate, iIvory),
	(17, 41) : (constColonizerDate, iHorse),
	(16, 42) : (constOldWorldDate, iIron),
	(28, 31) : (constOldWorldDate, iIron),
	(31, 10) : (constColonizerDate, iWine),
	(31, 6)  : (constColonizerDate, iSheep),
	(32, 11) : (constOldWorldDate, iIron),
	(36, 18) : (constColonizerDate, iCorn),
	(42, 18) : (constOldWorldDate, iFish),														
	#extra
	(40, 19)  : (constOldWorldDate,  iIron),   	# Rio
	(41, 21)  : (constOldWorldDate,  iAluminium),  # Bahia
	(61, 21)  : (constOldWorldDate,  iCotton),  # congo
	(100, 29) : (constOldWorldDate, iPearls),	#singapore
	(72, 35) : (constOldWorldDate, iPearls),	#medina or egypt
	(113, 43) : (constOldWorldDate, iAluminium),
	(110, 55) : (constOldWorldDate, iGems),
	(91, 54) : (constOldWorldDate, iCow),
	(69, 57) : (constOldWorldDate, iCow),
	(62, 54) : (constOldWorldDate, iCow),
	(61, 49) : (constOldWorldDate, iWine),
	(62, 52) : (constOldWorldDate, iDeer),
	(54, 50) : (constOldWorldDate, iWheat),
	(52, 55) : (constOldWorldDate, iPig),
	(53, 53) : (constOldWorldDate, iHorse),
	(51, 53) : (constOldWorldDate, iCopper),
	(29, 47) : (constOldWorldDate, iCopper),	# new jersey copper
	(118, 48) : (constOldWorldDate, iWhales),
	(116, 44) : (constOldWorldDate, iWhales),
	(106, 46) : (constOldWorldDate, iCrab),
	(98, 42) : (constOldWorldDate, iCow),
	(53, 41) : (constOldWorldDate, iCopper),
	(49, 53) : (constOldWorldDate, iCrab),
	(57, 55) : (constOldWorldDate, iWhales),	#amsterdam or hamburg
	(70, 41) : (constOldWorldDate, iClam),
	(70, 49) : (constOldWorldDate, iFish),
	(67, 56) : (constOldWorldDate, iFur),  # novgorod fur
	(66, 15) : (constColonizerDate, iCorn),
	(27, 27) : (constOldWorldDate, iBanana),
	(54, 44) : (constOldWorldDate, iHorse),	# barcelona
	(61, 40) : (constOldWorldDate, iMarble),
	(117, 49) : (constOldWorldDate, iCrab),
	(114, 44) : (constOldWorldDate, iRice),
	(115, 45) : (constOldWorldDate, iIncense),
	(106, 44) : (constOldWorldDate, iCopper),
	(79, 59) : (constOldWorldDate, iDeer),
	(82, 55) : (constOldWorldDate, iWheat),
	(108, 38) : (constOldWorldDate, iMarble),
	(69, 36) : (constOldWorldDate, iSugar),
	(101, 30) : (constOldWorldDate, iGold),
	(108, 28) : (constOldWorldDate, iGold),
	(101, 26) : (constOldWorldDate, iCoffee),
	(110, 35) : (constOldWorldDate, iFish),
	(30, 39) : (constOldWorldDate, iCoffee),
	(8, 48) : (constOldWorldDate, iWhales),	# san francisco
	(59, 25) : (constOldWorldDate, iFish),
	(63, 40) : (constOldWorldDate, iClam),
	(60, 41) : (constOldWorldDate, iFish),
	(56, 50) : (constOldWorldDate, iHorse),
	(70, 58) : (constOldWorldDate, iFish),
	(54, 54) : (constOldWorldDate, iFish),
	(104, 38) : (constOldWorldDate, iPearls),
	(106, 39) : (constOldWorldDate, iPearls),
	(103, 24) : (constOldWorldDate, iClam),
	(107, 26) : (constOldWorldDate, iCrab),
	(107, 59) : (constOldWorldDate, iDeer),
	(107, 61) : (constOldWorldDate, iOil), # Siberia
	(27, 37) : (constOldWorldDate, iPearls),
	(31, 38) : (constOldWorldDate, iPearls),
	(34, 34) : (constOldWorldDate, iPearls),
	(21, 41) : (constOldWorldDate, iCrab),
	(32, 51) : (constOldWorldDate, iIron),
	(33, 51) : (constOldWorldDate, iFur),
	(29, 49) : (constOldWorldDate, iIron),
	(78, 49) : (constOldWorldDate, iPearls),
	(43, 21) : (constOldWorldDate, iFish),
	(47, 42) : (constOldWorldDate, iWhales),
	(116, 46) : (constOldWorldDate, iSilk),
	(116, 48) : (constOldWorldDate, iStone),
	(12, 45)  : (constOldWorldDate,  iGold),    # California
	(25, 42)  : (constOldWorldDate,  iDye),    # Florida
	(94, 50)  : (constOldWorldDate,  iCamel),    # Urumqi
	(52, 57)  : (constOldWorldDate,  iCoal),    # Manchester
	(56, 48)  : (constOldWorldDate,  iCoal),    # Lyon
	(57, 50)  : (constOldWorldDate,  iWine),    # Champagne
	(81, 53)  : (constOldWorldDate,  iCoal),    # Ural
	(69, 51)  : (constOldWorldDate,  iPig),    # Kiev
	(72, 52)  : (constOldWorldDate,  iCoal),    # Kharkiv
	(81, 47)  : (constOldWorldDate,  iOil),    # Turkmenistan
	(92, 60)  : (constOldWorldDate,  iOil),    # Tomsk
	(87, 53) : (constOldWorldDate,  iWheat),	# Omsk
	(30, 20)  : (constOldWorldDate,  iTobacco),    # Bolivia coca
	(34, 17)  : (constColonizerDate,  iTea),    # Paraguay mate
	(61, 57)  : (constOldWorldDate,  iCrab),    # Denmark
	(14, 44)  : (constColonizerDate,  iWheat),    # Phoenix
	(29, 44)  : (constOldWorldDate,  iWhales),    # Washington
	(84, 36)  : (constOldWorldDate,  iCamel),    # karachi
	(85, 35)  : (constOldWorldDate,  iFish),    # karachi
	(29, 32)  : (constOldWorldDate,  iCorn),   # Caracas
	(29, 34)  : (constOldWorldDate,  iFish),    # Maracaibo
	(103, 41)  : (constOldWorldDate,  iRice),    # Wuhan
	(103, 42)  : (constOldWorldDate,  iGold),    # Wuhan
	(111, 42)  : (constOldWorldDate,  iCopper),    # Nagasaki
	(113, 45)  : (constOldWorldDate,  iTea),    # Kyoto
	(112, 44)  : (constOldWorldDate,  iGold),    # Kyoto
	(11, 43)  : (constOldWorldDate,  iFish),    # San Diego
	(10, 51)  : (constOldWorldDate,  iCrab),    # Seattle
	(116, 51)  : (constOldWorldDate,  iCoal),    # Sapporo
	(62, 43)  : (constOldWorldDate,  iMarble),    # Naples
	(51, 38)  : (constOldWorldDate,  iCamel),    # Marrakesh
	(108, 32)  : (constOldWorldDate,  iCopper),    # Manila Swap
	(108, 35)  : (constOldWorldDate,  iGold),    # Manila Swap
	(81, 41)  : (constOldWorldDate,  iSheep),    # Tehran
	(68, 50)  : (constOldWorldDate,  iCoal),    # Odessa
	(109, 31)  : (constOldWorldDate,  iSpices),    # Davao
	(31, 54)  : (constOldWorldDate,  iDeer),    # Montreal
	(26, 52)  : (constOldWorldDate,  iDeer),    # Toronto
	(111, 44)  : (constOldWorldDate,  iPearls),    # kyoto
	(93, 47)  : (constOldWorldDate,  iOil),    # china oil in xinjiang
	(98, 47)  : (constOldWorldDate,  iSheep),    # baotou
	(108, 51)  : (constOldWorldDate,  iAluminium),    # manchuria
	(65, 55)  : (constOldWorldDate,  iGold),    # baltic amber
	(72, 54)  : (constOldWorldDate,  iPig),    # moscow food
	(79, 44)  : (constOldWorldDate,  iFish),	# caspian food
	(73, 48)  : (constOldWorldDate,  iPearls),	# azov pearls
	(67, 51)  : (constOldWorldDate,  iDye),	# warsaw
	(65, 53)  : (constOldWorldDate,  iHorse),	# warsaw
	(32, 21)  : (constOldWorldDate,  iBanana),	# la paz banana
	(60, 55)  : (constOldWorldDate,  iClam),	# denmark and hamburg
	(10, 43)  : (constOldWorldDate,  iWhales),	# san diego
	(56, 60)  : (constOldWorldDate,  iWhales),	# west norway
	(27, 38)  : (constOldWorldDate,  iCopper),	# copper cuba
	(36, 49)  : (constOldWorldDate,  iWhales),	# halifax
	(29, 55)  : (constOldWorldDate,  iFur),	# quebec
	(54, 51)  : (constOldWorldDate,  iDye),	# north france
	(32, 49)  : (constOldWorldDate,  iFish),	# boston
	(32, 48)  : (constOldWorldDate,  iClam),	# boston
	(111, 10) : (constOldWorldDate,  iWhales),	# adelaide
	(53, 61)  : (constOldWorldDate,  iWhales),	# plymouth
	(66, 59)  : (constOldWorldDate,  iFur),	# helsinki
	(24, 22)  : (constOldWorldDate,  iWhales),	# whales lima
	(71, 62)  : (constOldWorldDate,  iWhales),	# whales arhangelsk
	(114, 57) : (constOldWorldDate,  iWhales),	# whales magadan
	(121, 54) : (constOldWorldDate,  iWhales),	# whales magadan
	(48, 52)  : (constOldWorldDate,  iWhales),	# whales plymouth
	(49, 49)  : (constOldWorldDate,  iWhales),	# whales brest
	(48, 47)  : (constOldWorldDate,  iWhales),	# whales cantabria
	(27, 7)   : (constOldWorldDate,  iWhales),	# whales chile
	(68, 48)  : (constOldWorldDate,  iWine),	# wine moldavia
	(76, 46)  : (constOldWorldDate,  iWine),	# wine georgia
	(121, 10) : (constOldWorldDate,  iCrab),	# crab new zealand
	(32, 53)  : (constOldWorldDate,  iFish),	# montreal
	(107, 17) : (constOldWorldDate,  iGold),	# australia
	(70, 54)  : (constOldWorldDate,  iIron),	# kiev
	(69, 60)  : (constOldWorldDate,  iCopper),	# st petersburg
	(11, 50)  : (constOldWorldDate,  iDeer),	# san francisco
	(12, 49)  : (constOldWorldDate,  iSilver),    # California
	(51, 45)  : (constOldWorldDate,  iPig),		# madrid
	(73, 50)  : (constOldWorldDate,  iCoal),		# rostov
	(107, 45) : (constOldWorldDate,  iFish),		# shanghai fish
	(109, 43) : (constOldWorldDate,  iWhales),		# shanghai whales
	(77, 55) : (constOldWorldDate,  iHorse),		# Kazan horses
	(91, 50) : (constOldWorldDate,  iCotton),		# Urumqi cotton
	(80, 29) : (constOldWorldDate,  iSpices),		# Salalah
	(119, 11) : (constOldWorldDate,  iWhales),		# sydney
	(43, 17) : (constOldWorldDate,  iWhales),		# 
	(36, 56) : (constOldWorldDate,  iWhales),		# 
	(105, 20) : (constOldWorldDate,  iOil),		# australia
	(32, 50) : (constOldWorldDate,  iDeer),		# boston
	
	(55, 43)  : (constOldWorldDate,  iPearls),	# mediteranean
	(57, 42)  : (constOldWorldDate,  iPearls),	# mediteranean
	(68, 47)  : (constOldWorldDate,  iPearls),	# mediteranean
	(61, 42)  : (constOldWorldDate,  iPearls),	# mediteranean
	(75, 30)  : (constOldWorldDate,  iPearls),	# mediteranean
	(80, 35)  : (constOldWorldDate,  iPearls),	# mediteranean
	
	(51, 60)  : (constOldWorldDate,  iAluminium),	# aluminum scotland (strategy + zinc)
	(32, 3)  : (constOldWorldDate,  iFur),		# patagonia
	
	(66, 49)  : (constOldWorldDate,  iGold),	# carpathian gold instead of iron
	(61, 50)  : (constOldWorldDate,  iSilver),	# austria
	(64, 56)  : (constOldWorldDate,  iPearls),	# baltic pearls (fish leads to too much pop)
	(102, 37)  : (constOldWorldDate, iCrab),	# hanoi
	(103, 36)  : (constOldWorldDate, iPearls),	# tonkin
	(47, 36)  : (constLateAncientDate, iSugar),	# practically whole economy of canaries
	(53, 29)  : (constOldWorldDate, iGold),	# mali gold
	(52, 26)  : (constOldWorldDate,  iGold),	# gold coast gold
	(59, 26)  : (constOldWorldDate, iOil),		# lagos oil
	(59, 27)  : (constOldWorldDate,  iGold),  # lagos gold
	
	(66, 21)  : (constOldWorldDate,  iCopper),  # congo
	
	(15, 48)  : (constColonizerDate,  iCow),  # las vegas
	(9, 47)  : (constOldWorldDate,  iClam),  # los angeles
	
	(29, 46)  : (constOldWorldDate,  iFish),  # philadelphia
	(31, 46)  : (constOldWorldDate,  iWhales),  # new york
	(29, 12)  : (constOldWorldDate,  iCopper),  # santiago
	(29, 10)  : (constOldWorldDate,  iDeer),  # santiago
	(29, 9)  : (constColonizerDate,  iWine),  # santiago
	
	(33, 14)  : (constColonizerDate,  iTea),  # asuncion mate tea
	
	(21, 58)  : (constOldWorldDate,  iGems),  # canada gems
	(15, 60)  : (constOldWorldDate,  iGems),  # canada gems
	
	(42, 27)  : (constOldWorldDate,  iWhales),  # fortaleza
	
	(59, 30) : (constOldWorldDate,  iCamel),   # kano
	(110, 43) : (constOldWorldDate,  iFish),   # fukuoka
	(82, 62) : (constOldWorldDate,  iCoal),   # novy port
	
	(111, 55) : (constOldWorldDate,  iDeer),   # khabarovsk
	(109, 59) : (constOldWorldDate,  iOil),   # Okhotsk
	
	(106, 55) : (constOldWorldDate,  iFur),   # Manchuria
	(115, 61) : (constOldWorldDate,  iFur),   # Magadan
	(93, 62) : (constOldWorldDate,  iFur),   # Khatanga
	(96, 63) : (constOldWorldDate,  iGems),   # Khatanga (popigai)
	(121, 63) : (constOldWorldDate,  iGems),   # Pevek quasicrystals
	(113, 55) : (constOldWorldDate,  iFur),	#khabarovsk
	
	(113, 17) : (constOldWorldDate,  iCopper),   # Australia
	(19, 34) : (constOldWorldDate,  iStone),   # Puebla, Maya
	(12, 41) : (constOldWorldDate,  iTobacco),   # la paz de california
	
	(59, 35) : (constOldWorldDate,  iUranium),   # uraniun lybia
	
	(31, 26) : (constOldWorldDate,  iBanana),   # amazon
	(103, 23) : (constOldWorldDate,  iWhales),   # jakarta
	(104, 39) : (constOldWorldDate,  iStone),   # guangzhou
	(108, 41) : (constOldWorldDate,  iClam),	  #hangzhou
	
	(109, 28) : (constOldWorldDate,  iSpices),   # makasaar
	(104, 28) : (constOldWorldDate,  iSpices),   # borneo
	
	(6, 19) : (constOldWorldDate,  iSugar),   # pacific sugar
	(3, 20) : (constOldWorldDate,  iPearls), 	# pacific, instead of fish
	
	(7, 57) : (constOldWorldDate,  iCrab),   # anchroage
	(6, 60) : (constOldWorldDate,  iFur), 
	(12, 57) : (constOldWorldDate,  iSilver), 	# north cascadia silver
	
	(27, 46) : (constOldWorldDate,  iHorse),   # washington
	(26, 45) : (constOldWorldDate,  iTobacco),
	
	(25, 50) : (constOldWorldDate,  iFur), #detroit fur trade
	
	(15, 37) : (constOldWorldDate,  iClam), #gaudalajara
	
	(76, 33) : (constOldWorldDate,  iSheep),	#arabia 
	(76, 34) : (constOldWorldDate,  iHorse),
	(75, 35) : (constOldWorldDate,  iCamel),
	
	(63, 46) : (constOldWorldDate,  iCopper),	# balkans
	(66, 62) : (constOldWorldDate,  iFur),		# scandinavia
	(62, 59) : (constOldWorldDate,  iCopper),	# scandinavia, falun mine.
	
	(64, 35) : (constOldWorldDate,  iCamel),	# lybia camel
}

# TODO: should be handled by Rise and Fall
# TODO: Rise and Fall needs generic function canSpawn for normal civs and rebirths
dSpawnResourcesDict = {
	(90, 28) : (iTamils,    iFish),
	(95, 43) : (iTibet,     iWheat),
	(97, 44) : (iTibet,     iHorse),
	(78, 51) : (iMongols,   iSilk),
	(61, 22) : (iCongo,     iCotton),
	(63, 19) : (iCongo,     iIvory),
	(61, 24) : (iCongo,     iIvory),
	(17, 42) : (iMexico,    iHorse),
	(16, 42) : (iMexico,    iIron),
	(28, 31) : (iColombia,  iIron),
	(31, 10) : (iArgentina, iWine),
	(31, 6)  : (iArgentina, iSheep),
	(32, 11) : (iArgentina, iIron),
	(36, 18) : (iBrazil,    iCorn),
	(42, 18) : (iBrazil,    iFish),
}

dRemovedResourcesDict = {
	(51, 36) : constLateAncientDate, # Ivory in Morocco
	(58, 37) : constLateAncientDate, # Ivory in Tunisia
	(87, 49) : constLateAncientDate, # Sheep near Orduqent
	(89, 51) : constLateAncientDate, # Camel near Orduqent
	(63, 66) : constOldWorldDate, # north norway whales
	(45, 61) : constOldWorldDate, # iceland fish
	(51, 62) : constOldWorldDate, # scotland whales
	(30, 53) : constOldWorldDate, # quebec fur
	(113, 7) : constOldWorldDate, # tasmania whales
	(49, 57) : constOldWorldDate, # ireland horse
	(23, 22) : constOldWorldDate, # whales quito
	(26, 19) : constOldWorldDate, # whales lima
	(119, 52) : constOldWorldDate, # whales kamchatka
	(121, 8) : constOldWorldDate, # crab new zealand
	(69, 50) : constOldWorldDate, # iron ukraine
	(67, 57) : constOldWorldDate, # cow estonia
	(68, 60) : constOldWorldDate, # copper helsinki
	(50, 47) : constOldWorldDate, # crab cantabria
	(11, 49) : constOldWorldDate, # deer san francisco
	(66, 53) : constOldWorldDate, # horse warsaw
	(74, 50) : constOldWorldDate, # coal rostov
	(92, 50) : constOldWorldDate, # cotton urumqi
	(22, 38) : constOldWorldDate, # crab maya
	(38, 55) : constOldWorldDate, 
	(65, 59) : constOldWorldDate, 
	(102, 19) : constOldWorldDate, 
	(101, 17) : constOldWorldDate, 
	(61, 22) : constOldWorldDate, #congo cotton
	(53, 28) : constOldWorldDate, #mali gold
	(51, 28) : constOldWorldDate, # medina
	(60, 26) : constOldWorldDate, #lagos oil
	(66, 20) : constOldWorldDate, #congo copper
	(10, 47) : constOldWorldDate, #clam
	(30, 46) : constOldWorldDate, #fish
	(108, 40) : constOldWorldDate, # claim taibei
	
	(17, 62) : constOldWorldDate, #gems canada
	(19, 61) : constOldWorldDate, #gems canada
	(39, 29) : constOldWorldDate, #whales macapa
	(106, 62) : constOldWorldDate, #sakha fur
	(107, 61) : constOldWorldDate, #sakha oil
	
	(105, 55) : constOldWorldDate, #manchuria fur
	(113, 18) : constOldWorldDate, #australia copper
	
	(19, 35) : constOldWorldDate, #stone maya
	(59, 34) : constOldWorldDate, #uranium lybia
	
	(29, 9) : constOldWorldDate,	#santiago deer
	(28, 3) : constOldWorldDate, #tierra del fuego						   
	
	(111, 27) : constOldWorldDate, #papua new guinea	
	(113, 27) : constOldWorldDate,
	
	(2, 20) : constOldWorldDate,	# pacific sugar
	(9, 57) : constOldWorldDate,	# alaska crab
	
	(27, 45) : constOldWorldDate, # washington
	(33, 49) : constOldWorldDate, # boston
	(32, 52) : constOldWorldDate, # boston
	
	(14, 40) : constOldWorldDate, # sea of cortez
	
	(12, 58) : constOldWorldDate, # alaska silver
	(14, 51) : constOldWorldDate, # montana silver
	
	(76, 32) : constOldWorldDate, # arabian mountain range
	(74, 34) : constOldWorldDate, # arabian mountain range
	(74, 35) : constOldWorldDate, # medina
	(112, 41) : constOldWorldDate, # kagoshima
	(64, 45) : constOldWorldDate, # balkans copper
	(67, 63) : constOldWorldDate, # fur scandinavia
	(62, 57) : constOldWorldDate, # copper scandinavia
	(71, 28) : constOldWorldDate, # ethiopia camel
	
	# 1700 map
	(26, 49) : constOldWorldDate, # detroit pig
	(61, 22) : constOldWorldDate, # congo cotton
}

dRoutesDict = {
	constLateAncientDate : lSilkRoute,
	constLateAncientDate : [(88, 47)],
}

dSpawnRoutes = {
	iMongols : [(101, 48), (100, 49), (100, 50), (99, 50)],
}

# there must be stuff like this elsewhere, maybe barbs?
dPlotTypesDict = {
	(88, 47) : (constLateAncientDate, PlotTypes.PLOT_HILLS),
	
	(25, 25) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#various
	(27, 29) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(31, 20) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(112, 26) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(28, 32) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(25, 28) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(108, 35) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(115, 51) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(114, 51) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(108, 36) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(115, 7) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(115, 8) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(23, 50) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(107, 44) : (constPlotChangeDate, PlotTypes.PLOT_LAND),
	(72, 23) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#mombasa
	(71, 23) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#mombasa
	(47, 63) : (constPlotChangeDate, PlotTypes.PLOT_LAND),	#iceland
	(29, 5) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	# south chile
	(65, 42) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#greece
	(72, 32) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#ethiopia
	(72, 31) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#ethiopia
	(71, 31) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#ethiopia
	(71, 30) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#ethiopia
	(72, 26) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#ethiopia
	(70, 25) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#ethiopia
	
	(10, 50) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	# california enlargement + interior valley
	(10, 49) : (constPlotChangeDate, PlotTypes.PLOT_LAND),
	(10, 48) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(10, 47) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	
	(13, 41) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	# baja california
	
	(29, 12) : (constPlotChangeDate, PlotTypes.PLOT_HILLS), # santiago
	(7, 59) : (constPlotChangeDate, PlotTypes.PLOT_HILLS), # alaska
	(11, 60) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	# no more whitehorse
	
	(76, 32) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#arabian mountain range
	(74, 34) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(73, 35) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	
	(54, 37) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#atlas
	(51, 29) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#west africa
	
	(36, 1) : (constPlotChangeDate, PlotTypes.PLOT_OCEAN),	#south passage
	
	(16, 50) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#wyoming
	(82, 43) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#iran
	(64, 45) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#balkans
	(65, 44) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#balkans
	(63, 46) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#balkans
	
	(67, 64) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#scandinavia
	(66, 64) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#scandinavia
	(66, 65) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#scandinavia
	(65, 65) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#scandinavia
}


dFeaturesDict = {
	(35, 54) : (constLateAncientDate,  iMud),         # Newfoundland obstacles
	(92, 35) : (constOldWorldDate, iRainforest),  # Deccan
	(11, 46) : (constOldWorldDate, iFloodPlains), # California
	(11, 47) : (constOldWorldDate, iFloodPlains), # California
	(11, 48) : (constOldWorldDate, iFloodPlains), # California
	
	(10, 50) : (constOldWorldDate, iForest),	#california forests
	(10, 47) : (constOldWorldDate, iForest),	#california forests
	
	(107, 44): (constOldWorldDate, iFloodPlains),
	(106, 44): (constOldWorldDate, iFloodPlains),
	(105, 45): (constOldWorldDate, iFloodPlains),
	(53, 54) : (constOldWorldDate, iFloodPlains),
	(56, 49) : (constOldWorldDate, iFloodPlains),
	(85, 37) : (constOldWorldDate, iFloodPlains),
	(56, 46) : (constOldWorldDate, iFloodPlains),
	(62, 42) : (constOldWorldDate, iFloodPlains),
	(66, 53) : (constOldWorldDate, iFloodPlains),
	(58, 47) : (constOldWorldDate, iFloodPlains),
	(59, 47) : (constOldWorldDate, iFloodPlains),
	(55, 50) : (constOldWorldDate, iFloodPlains),
	(62, 53) : (constOldWorldDate, iFloodPlains),
	(62, 51) : (constOldWorldDate, iFloodPlains),
	(62, 49) : (constOldWorldDate, iFloodPlains),
	(59, 54) : (constOldWorldDate, iFloodPlains),
	(60, 40) : (constOldWorldDate, iFloodPlains),
	(52, 43) : (constOldWorldDate, iFloodPlains),
	(51, 43) : (constOldWorldDate, iFloodPlains),
	(56, 48) : (constOldWorldDate, iFloodPlains),
	(56, 31) : (constOldWorldDate, iFloodPlains),
	(55, 44) : (constOldWorldDate, iFloodPlains),
	(69, 52) : (constOldWorldDate, iFloodPlains),
	(72, 43) : (constOldWorldDate, iFloodPlains),
	(69, 30) : (constOldWorldDate, iFloodPlains),
	(26, 29) : (constOldWorldDate, iFloodPlains),
	(67, 14) : (constOldWorldDate, iFloodPlains),
	(38, 22) : (constOldWorldDate, iFloodPlains),
	(28, 28) : (constOldWorldDate, iFloodPlains),
	(17, 36) : (constOldWorldDate, iFloodPlains),
	(27, 29) : (constOldWorldDate, iFloodPlains),
	(19, 38) : (constOldWorldDate, iFloodPlains),
	(16, 39) : (constOldWorldDate, iFloodPlains),
	(73, 54) : (constOldWorldDate, iFloodPlains),
	(15, 45) : (constOldWorldDate, iFloodPlains),
	(31, 48) : (constOldWorldDate, iFloodPlains),		# new york
	(34, 11) : (constOldWorldDate, iFloodPlains),		# buenos aires
	(52, 44) : (constOldWorldDate, iFloodPlains),		# madrid
	(41, 18) : (constOldWorldDate, iFloodPlains),		# rio
	(72, 30) : (constOldWorldDate, iFloodPlains), 		# ethiopia 
	
	(23, 49) : (constOldWorldDate, iFloodPlains),
	(30, 47) : (constOldWorldDate, iFloodPlains),
	(30, 48) : (constOldWorldDate, iFloodPlains),
	(31, 49) : (constOldWorldDate, iFloodPlains),
	(28, 49) : (constOldWorldDate, iFloodPlains),
	(113, 45) : (constOldWorldDate, iFloodPlains),
	(116, 47) : (constOldWorldDate, iFloodPlains),
	(115, 45) : (constOldWorldDate, iFloodPlains),
	(109, 46) : (constOldWorldDate, iFloodPlains),
	(39, 17) : (constOldWorldDate, iFloodPlains),
	(40, 17) : (constOldWorldDate, iFloodPlains),
	(39, 18) : (constOldWorldDate, iFloodPlains),
	(38, 18) : (constOldWorldDate, iFloodPlains),
	(102, 46) : (constOldWorldDate, iFloodPlains),
	(106, 43) : (constOldWorldDate, iFloodPlains),
	(105, 43) : (constOldWorldDate, iFloodPlains),
	(107, 43) : (constOldWorldDate, iFloodPlains),
	(80, 40) : (constOldWorldDate, iFloodPlains),
	(80, 42) : (constOldWorldDate, iFloodPlains),
	(78, 43) : (constOldWorldDate, iFloodPlains),
	(88, 49) : (constOldWorldDate, iFloodPlains),
	(99, 47) : (constOldWorldDate, iFloodPlains),
	(102, 47) : (constOldWorldDate, iFloodPlains),
	(100, 44) : (constOldWorldDate, iFloodPlains),
	(99, 41) : (constOldWorldDate, iFloodPlains),
	(101, 41) : (constOldWorldDate, iFloodPlains),
	(101, 39) : (constOldWorldDate, iFloodPlains),	
	
	(89, 51) : (constOldWorldDate, iFloodPlains),
	(69, 36) : (constOldWorldDate, iFloodPlains),
	(71, 28) : (constOldWorldDate, iFloodPlains),
	(71, 45) : (constOldWorldDate, iFloodPlains),
	(61, 47) : (constOldWorldDate, iFloodPlains),
	(101, 33) : (constOldWorldDate, iFloodPlains),
	(101, 34) : (constOldWorldDate, iFloodPlains),
	(101, 37) : (constOldWorldDate, iFloodPlains),
	(92, 32) : (constOldWorldDate, iFloodPlains),
	(88, 34) : (constOldWorldDate, iFloodPlains),
	(89, 35) : (constOldWorldDate, iFloodPlains),
	(79, 43) : (constOldWorldDate, iFloodPlains),
	(54, 42) : (constOldWorldDate, iFloodPlains),
	(100, 47) : (constOldWorldDate, iFloodPlains),
	(101, 42) : (constOldWorldDate, iFloodPlains),
	(108, 34) : (constOldWorldDate, iFloodPlains),
	(104, 25) : (constOldWorldDate, iFloodPlains),
	(106, 25) : (constOldWorldDate, iFloodPlains),
	(91, 34) : (constOldWorldDate, iFloodPlains),
	(90, 31) : (constOldWorldDate, iFloodPlains),
	(92, 32) : (constOldWorldDate, iFloodPlains),
	(15, 44) : (constOldWorldDate, iFloodPlains),
	(13, 44) : (constOldWorldDate, iFloodPlains),
	(60, 50) : (constOldWorldDate, iFloodPlains),
	(59, 53) : (constOldWorldDate, iFloodPlains),
	(61, 53) : (constOldWorldDate, iFloodPlains),
	(56, 50) : (constOldWorldDate, iFloodPlains),
	(62, 50) : (constOldWorldDate, iFloodPlains),
	(105, 39) : (constOldWorldDate, iFloodPlains),
	(102, 42) : (constOldWorldDate, iFloodPlains),
	(103, 42) : (constOldWorldDate, iFloodPlains),
	(103, 43) : (constOldWorldDate, iFloodPlains),
	(55, 46) : (constOldWorldDate, iFloodPlains),
	(55, 45) : (constOldWorldDate, iFloodPlains),
	(91, 47) : (constOldWorldDate, iFloodPlains),
	(89, 50) : (constOldWorldDate, iFloodPlains),
	(99, 43) : (constOldWorldDate, iFloodPlains),
	(31, 31) : (constOldWorldDate, iFloodPlains),
	(18, 40) : (constOldWorldDate, iFloodPlains),	#monterrey
	(53, 53) : (constOldWorldDate, iFloodPlains),	# London
	(99, 40) : (constOldWorldDate, iFloodPlains),	# Chengdu
	(60, 46) : (constOldWorldDate, iFloodPlains),	# Po valley
	(14, 49) : (constOldWorldDate, iFloodPlains),	# salt lake city
	
	(93, 50) : (constOldWorldDate, iOasis),
	(50, 36) : (constOldWorldDate, iOasis),
	(110, 40) : (constOldWorldDate, iForest),
	(81, 44) : (constOldWorldDate, iOasis),		#Tehran
	
	(58, 44) : (constOldWorldDate, iIslands),		# corsica
	(51, 52) : (constOldWorldDate, iIslands),		# isle jersey
	(64, 58) : (constOldWorldDate, iIslands),		# gotland
	(47, 35) : (constOldWorldDate, iIslands),		# canaries
	(113, 44) : (constOldWorldDate, iIslands),
	(112, 43) : (constOldWorldDate, iIslands),
	(111, 41) : (constOldWorldDate, iIslands),
	(62, 39) : (constOldWorldDate, iIslands),
	(64, 42) : (constOldWorldDate, iIslands),
	(51, 57) : (constOldWorldDate, iIslands),
	(61, 56) : (constOldWorldDate, iIslands),
	(65, 57) : (constOldWorldDate, iIslands),
	(62, 46) : (constOldWorldDate, iIslands),
	(27, 32) : (constOldWorldDate, iIslands),
	(109, 39) : (constOldWorldDate, iIslands),
	(107, 29) : (constOldWorldDate, iIslands),
	(102, 25) : (constOldWorldDate, iIslands),
	(97, 29) : (constOldWorldDate, iIslands),
	(98, 27) : (constOldWorldDate, iIslands),
	(107, 25) : (constOldWorldDate, iIslands),
	(29, 33) : (constOldWorldDate, iIslands),
	(25, 40) : (constOldWorldDate, iIslands),
	(28, 37) : (constOldWorldDate, iIslands),
	(30, 47) : (constOldWorldDate, iIslands),
	(63, 55) : (constOldWorldDate, iIslands),
	(63, 45) : (constOldWorldDate, iIslands),
	(69, 40) : (constOldWorldDate, iIslands),
	(110, 44) : (constOldWorldDate, iIslands),	#korea
	(113, 42) : (constOldWorldDate, iIslands),	#kagoshima
	
	(105, 38) : (constOldWorldDate, iIslands),	#guangzhou
	(106, 37) : (constOldWorldDate, iIslands),
	
	(23, 34) : (constOldWorldDate, iRainforest),	#maya
	
	(64, 22) : (constOldWorldDate, iRainforest),	#kinshasa
	(27, 38) : (constOldWorldDate, iRainforest),
	(33, 31) : (constOldWorldDate, iRainforest),
	(35, 29) : (constOldWorldDate, iRainforest),
	(108, 35) : (constOldWorldDate, iRainforest),
	(109, 31) : (constOldWorldDate, iRainforest),
	(108, 30) : (constOldWorldDate, iJungle),		# philippines
	(110, 32) : (constOldWorldDate, iForest),
	(32, 21) : (constOldWorldDate, iRainforest),
	
	(18, 50) : (constOldWorldDate, iForest),		# china
	(17, 51) : (constOldWorldDate, iForest),
	(108, 38) : (constOldWorldDate, iForest),
	
	(79, 43) : (constOldWorldDate, iForest),		# iran
	(77, 44) : (constOldWorldDate, iForest),
	
	(66, 57) : (constOldWorldDate, iForest),		# europe
	(49, 58) : (constOldWorldDate, iForest),
	(49, 56) : (constOldWorldDate, iForest),
	(67, 57) : (constOldWorldDate, iForest),
	(61, 54) : (constOldWorldDate, iForest),		
	(61, 52) : (constOldWorldDate, iForest),
	(58, 53) : (constOldWorldDate, iForest),
	(57, 51) : (constOldWorldDate, iForest),		
	(65, 56) : (constOldWorldDate, iForest),
	(63, 54) : (constOldWorldDate, iForest),
	(64, 50) : (constOldWorldDate, iForest),
	(66, 55) : (constOldWorldDate, iForest),
	(67, 50) : (constOldWorldDate, iForest),
	(52, 53) : (constOldWorldDate, iForest),
	(57, 49) : (constOldWorldDate, iForest),
	
	(76, 48) : (constOldWorldDate, iForest),		#rus
	(78, 58) : (constOldWorldDate, iForest),
	(112, 44) : (constOldWorldDate, iForest),
	(107, 41) : (constOldWorldDate, iForest),
	(106, 45) : (constOldWorldDate, iForest),
	(68, 51) : (constOldWorldDate, iForest),
	(77, 54) : (constOldWorldDate, iForest),
	(76, 53) : (constOldWorldDate, iForest),
	(79, 57) : (constOldWorldDate, iForest),
	
	(90, 56) : (constOldWorldDate, iForest),		#siberia
	(84, 57) : (constOldWorldDate, iForest),
	(96, 57) : (constOldWorldDate, iForest),
	(97, 56) : (constOldWorldDate, iForest),
	(98, 55) : (constOldWorldDate, iForest),
	(105, 57) : (constOldWorldDate, iForest),
	(101, 54) : (constOldWorldDate, iForest),
	(103, 56) : (constOldWorldDate, iForest),
	
	(25, 50) : (constOldWorldDate, iForest),		# usa
	(25, 49) : (constOldWorldDate, iForest),
	(23, 50) : (constOldWorldDate, iForest),
	(22, 50) : (constOldWorldDate, iForest),
	(28, 48) : (constOldWorldDate, iForest),
	(13, 51) : (constOldWorldDate, iForest),
	(19, 53) : (constOldWorldDate, iForest),
	(24, 48) : (constOldWorldDate, iForest),
	(25, 42) : (constOldWorldDate, iRainforest),
	(30, 37) : (constOldWorldDate, iRainforest),
	(77, 56) : (constOldWorldDate, iForest),	
	(73, 59) : (constOldWorldDate, iForest),	
	(70, 61) : (constOldWorldDate, iForest),	
	(66, 59) : (constOldWorldDate, iForest),	
	(70, 63) : (constOldWorldDate, iForest),	
	(24, 43) : (constOldWorldDate, iForest),	
	(29, 52) : (constOldWorldDate, iForest),	
	(27, 50) : (constOldWorldDate, iForest),	
	(28, 46) : (constOldWorldDate, iForest),	
	(26, 45) : (constOldWorldDate, iForest),	
	(27, 47) : (constOldWorldDate, iForest),	
	
	
	(51, 28) : (constOldWorldDate, iJungle),		# ghana and congo
	(66, 22) : (constOldWorldDate, iJungle),
	(64, 19) : (constOldWorldDate, iJungle),
	(67, 21) : (constOldWorldDate, iJungle),
	(66, 19) : (constOldWorldDate, iJungle),
	(55, 27) : (constOldWorldDate, iJungle),
	(49, 29) : (constOldWorldDate, iJungle),
	(54, 28) : (constOldWorldDate, iJungle),
	(52, 26) : (constOldWorldDate, iRainforest),
	(54, 29) : (constOldWorldDate, iRainforest),
	(55, 28) : (constOldWorldDate, iJungle),
	(55, 29) : (constOldWorldDate, iRainforest),
	(56, 28) : (constOldWorldDate, iRainforest),
	(57, 27) : (constOldWorldDate, iRainforest),
	(97, 40) : (constOldWorldDate, iJungle),
	(97, 38) : (constOldWorldDate, iJungle),
	(64, 19) : (constOldWorldDate, iMud),
	(66, 20) : (constOldWorldDate, iJungle),
	(66, 21) : (constOldWorldDate, iRainforest),
	(63, 18) : (constOldWorldDate, iRainforest),
	(98, 38) : (constOldWorldDate, iJungle),
	(112, 26) : (constOldWorldDate, iJungle),
	(61, 22) : (constOldWorldDate, iJungle),		# congo jungle
	(53, 28) : (constOldWorldDate, iJungle),		# ghana
	(53, 29) : (constOldWorldDate, iRainforest),		# ghana
	
	(60, 23) : (constOldWorldDate, iRainforest),
	(115, 33) : (constOldWorldDate, iRainforest),
	(119, 27) : (constOldWorldDate, iRainforest),
	(122, 19) : (constOldWorldDate, iRainforest),
	(32, 31) : (constOldWorldDate, iRainforest),
	(30, 39) : (constOldWorldDate, iRainforest),
	(27, 36) : (constOldWorldDate, iRainforest),
	(36, 28) : (constOldWorldDate, iRainforest),
	(53, 26) : (constOldWorldDate, iRainforest),	# gold coast
	
	(47, 63) : (constOldWorldDate,  iMud),         # Iceland has too many cities and too much pop
	(45, 63) : (constOldWorldDate,  iMud),         # Iceland has too many cities and too much pop
	
	(54, 36) : (constOldWorldDate, iOasis),		# sahara morocco
	(56, 38) : (constOldWorldDate, iOasis),		# algiers
	(60, 30) : (constOldWorldDate, iOasis),		# kano
	(55, 34) : (constOldWorldDate, iOasis), 		# tamanrasset
	
	(35, 17) : (constOldWorldDate, iJungle),		# south america needs more jungle and rainforest
	(25, 32) : (constOldWorldDate, iRainforest),
	(25, 28) : (constOldWorldDate, iRainforest),
	(33, 15) : (constOldWorldDate, iJungle),
	(39, 22) : (constOldWorldDate, iJungle),
	(40, 23) : (constOldWorldDate, iJungle),
	(33, 29) : (constOldWorldDate, iJungle),
	(34, 28) : (constOldWorldDate, iJungle),
	(37, 26) : (constOldWorldDate, iRainforest),
	
	(36, 17) : (constOldWorldDate, iRainforest),
	(37, 28) : (constOldWorldDate, iRainforest),
	(41, 25) : (constOldWorldDate, iRainforest),
	(40, 24) : (constOldWorldDate, iRainforest),
	(42, 24) : (constOldWorldDate, iRainforest),
	(34, 18) : (constOldWorldDate, iRainforest),
	
	(33, 27) : (constOldWorldDate, iRainforest),	# make inner brasil settleable
	(35, 21) : (constOldWorldDate, iRainforest),
	(32, 24) : (constOldWorldDate, iRainforest),
	
	(104, 37) : (constOldWorldDate, iRainforest),	# hainan
	
	(109, 51) : (constOldWorldDate, iMud), 	# vladivostok
	
	(110, 16) : (constOldWorldDate, iOasis), # alice springs
	
	(113, 27) : (constOldWorldDate, iJungle),	#papua new guinea more jungle
	(111, 27) : (constOldWorldDate, iJungle),
	
	(29, 5) : (constOldWorldDate, iForest),	# horn cape
	
	(63, 31) : (constOldWorldDate, iOasis),	# sahara oasis
	(82, 43) : (constOldWorldDate, iForest),# iran
	
	(64, 46) : (constOldWorldDate, iForest),# balkans
	(63, 46) : (constOldWorldDate, iForest),# balkans
	
	(67, 63) : (constOldWorldDate, iMud),	# scandinavia
	(59, 57) : (constOldWorldDate, iForest),# scandinavia
	
	(72, 23) : (constOldWorldDate, iRainforest),# east africa, acts as military unit blocker between ethiopia and the rest of east africa
}

dRemovedFeaturesDict = {
	(67, 30)  : constLateAncientDate,  # Sudan
	(67, 31)  : constLateAncientDate,  # Sudan
	(113, 25) : constOldWorldDate, # allow settling New Guinea
	(102, 35) : constColonizerDate, # open up Vietnam
	(35, 54)  : constColonizerDate, # Newfoundland blocker
	(116, 24) : constColonizerDate, # Port Moresby
	(82, 47)  : constColonizerDate, # Transoxiana
	(83, 46)  : constColonizerDate, # Transoxiana
	(85, 49)  : constColonizerDate, # Transoxiana
	(114, 51)  : constOldWorldDate,  # Sapporo
	(56, 35)  : constOldWorldDate,  # sahara oasis
	(53, 34)  : constOldWorldDate,  # sahara oasis
	(34, 15)  : constOldWorldDate,  # asuncion							 
	(120, 64)  : constOldWorldDate,  # bering ice
	(121, 64)  : constOldWorldDate,  # bering ice
	(122, 64)  : constOldWorldDate,  # bering ice
	(8, 66)  : constOldWorldDate,  # alaska ice
	(9, 66)  : constOldWorldDate,  # alaska ice
	(27, 45)  : constOldWorldDate,  # forest washington
	(28, 47)  : constOldWorldDate,  # forest philadelphia
	(64, 31)  : constOldWorldDate,  # sahara oasis
	(64, 45)  : constOldWorldDate,  # balkans
	(65, 44)  : constOldWorldDate,  # balkans
	(67, 64)  : constOldWorldDate,  # scandinavia
	(66, 64)  : constOldWorldDate,  # scandinavia
	(66, 65)  : constOldWorldDate,  # scandinavia
	(65, 65)  : constOldWorldDate,  # scandinavia
	(54, 44)  : constOldWorldDate,  # barcelona
}

@handler("BeginGameTurn")
def changePlotType():
	# retroactively apply map changes on scenario start
	if game.getGameTurn() == scenarioStartTurn() and game.getGameTurn() >= year(constPlotChangeDate):
		for tile, type in dPlotTypes[year(constPlotChangeDate)]:
			plot(tile).setPlotType(type, True, True)
	if (game.getGameTurn() == year(constOldWorldDate)) or (game.getGameTurn() == year(constColonizerDate)) or (game.getGameTurn() == year(constLateAncientDate)):
		for tile, type in dPlotTypes[game.getGameTurn()]:
			plot(tile).setPlotType(type, True, True)

@handler("BeginGameTurn")
def createResources():
	# retroactively apply resource on scenario start
	if game.getGameTurn() == scenarioStartTurn() and game.getGameTurn() >= year(constOldWorldDate):
		for (x, y), iResource in dResources[year(constOldWorldDate)]:
			createResource(x, y, iResource)
	if game.getGameTurn() == scenarioStartTurn() and game.getGameTurn() >= year(constColonizerDate):
		for (x, y), iResource in dResources[year(constColonizerDate)]:
			createResource(x, y, iResource)
	if (game.getGameTurn() == year(constOldWorldDate)) or (game.getGameTurn() == year(constColonizerDate)) or (game.getGameTurn() == year(constLateAncientDate)):
		for (x, y), iResource in dResources[game.getGameTurn()]:
			createResource(x, y, iResource)


@handler("BeginGameTurn")
def createResourcesBeforeSpawn(iGameTurn):
	for iCiv in dSpawnResources:
		if iGameTurn == year(dBirth[iCiv]) - 1 and data.isCivEnabled(iCiv):
			for (x, y), iResource in dSpawnResources[iCiv]:
				createResource(x, y, iResource)


@handler("BeginGameTurn")
def removeResources():
	# retroactively apply resource on scenario start (only old world date is relevant here)
	if game.getGameTurn() == scenarioStartTurn() and game.getGameTurn() >= year(constOldWorldDate):
		for x, y in dRemovedResources[year(constOldWorldDate)]:
			removeResource(x, y)
	if (game.getGameTurn() == year(constOldWorldDate)) or (game.getGameTurn() == year(constColonizerDate)) or (game.getGameTurn() == year(constLateAncientDate)):
		for x, y in dRemovedResources[game.getGameTurn()]:
			removeResource(x, y)


@handler("BeginGameTurn")
def createRoutes():
	if (game.getGameTurn() == year(constOldWorldDate)) or (game.getGameTurn() == year(constColonizerDate)) or (game.getGameTurn() == year(constLateAncientDate)):
		for tile in dRoutes[game.getGameTurn()]:
			plot(tile).setRouteType(iRouteRoad)


@handler("BeginGameTurn")
def createRoutesBeforeSpawn(iGameTurn):
	for iCiv in dSpawnRoutes:
		if iGameTurn == year(dBirth[iCiv]) - 2 and data.isCivEnabled(iCiv):
			for tile in dSpawnRoutes[iCiv]:
				plot(tile).setRouteType(iRouteRoad)


@handler("BeginGameTurn")
def createFeatures():
	# retroactively create features
	if game.getGameTurn() == scenarioStartTurn() and game.getGameTurn() >= year(constOldWorldDate):
		for tile, iFeature in dFeatures[year(constOldWorldDate)]:
			plot(tile).setFeatureType(iFeature, 0)
	if (game.getGameTurn() == year(constOldWorldDate)) or (game.getGameTurn() == year(constColonizerDate)) or (game.getGameTurn() == year(constLateAncientDate)):
		for tile, iFeature in dFeatures[game.getGameTurn()]:
			plot(tile).setFeatureType(iFeature, 0)


@handler("BeginGameTurn")
def removeFeatures(iGameTurn):
	# retroactively remove features
	if game.getGameTurn() == scenarioStartTurn() and game.getGameTurn() >= year(constOldWorldDate):
		for tile in dRemovedFeatures[year(constOldWorldDate)]:
			plot(tile).setFeatureType(-1, 0)

	if (game.getGameTurn() == year(constOldWorldDate)) or (game.getGameTurn() == year(constColonizerDate)) or (game.getGameTurn() == year(constLateAncientDate)):
		for tile in dRemovedFeatures[game.getGameTurn()]:
			plot(tile).setFeatureType(-1, 0)
		
	if iGameTurn == year(700) and player(iVikings).isHuman():
		plot(41, 58).setFeatureType(-1, 0)
		

# Leoreth: bonus removal alerts by edead
def createResource(iX, iY, iBonus, createTextKey="TXT_KEY_MISC_DISCOVERED_NEW_RESOURCE", removeTextKey="TXT_KEY_MISC_EVENT_RESOURCE_EXHAUSTED"):
	"""Creates a bonus resource and alerts the plot owner"""
	plot = plot_(iX, iY)
	
	iRemovedBonus = plot.getBonusType(-1) # for alert
	
	if iRemovedBonus == iBonus:
		return
	
	plot.setBonusType(iBonus)
			
	if iBonus == -1:
		iImprovement = plot.getImprovementType()
		if iImprovement >= 0:
			if infos.improvement(iImprovement).isImprovementBonusTrade(iRemovedBonus):
				plot.setImprovementType(-1)
		
	iOwner = plot.getOwner()
	if iOwner >= 0: # only show alert to the tile owner
		bWater = plot.isWater()
		city = closestCity(plot, iOwner, same_continent=not bWater, coastal_only=bWater)
		
		if iRemovedBonus >= 0:
			notifyResource(iOwner, city, iX, iY, iRemovedBonus, removeTextKey)
		
		if iBonus >= 0:
			notifyResource(iOwner, city, iX, iY, iBonus, createTextKey)


def notifyResource(iPlayer, city, iX, iY, iBonus, textKey):
	if not city: return
	
	if infos.bonus(iBonus).getTechReveal() == -1 or team(iPlayer).isHasTech(infos.bonus(iBonus).getTechReveal()):
		message(iPlayer, textKey, infos.bonus(iBonus).getText(), city.getName(), event=InterfaceMessageTypes.MESSAGE_TYPE_MINOR_EVENT, button=infos.bonus(iBonus).getButton(), location=(iX, iY))


def removeResource(iX, iY):
	"""Removes a bonus resource and alerts the plot owner"""
	if plot(iX, iY).getBonusType(-1) == -1: return
	createResource(iX, iY, -1)